@@ -26,15 +26,12 @@
 #include "level.h"
 #include "array.h"
 #include "dir.h"
-#include "speed.h"
 
 #include "game_server.h"
 #include "game_client.h"
 #include "game_proxy.h"
 #include "game_common.h"
 
-/*---------------------------------------------------------------------------*/
-
 #define DEMO_MAGIC (0xAF | 'N' << 8 | 'B' << 16 | 'R' << 24)
 #define DEMO_VERSION 9
 
@@ -44,27 +41,43 @@ fs_file demo_fp;
 
 /*---------------------------------------------------------------------------*/
 
+static const char *demo_path(const char *name)
+{
+    static char path[MAXSTR];
+    sprintf(path, "Replays/%s.nbr", name);
+    return path;
+}
+
+static const char *demo_name(const char *path)
+{
+    static char name[MAXSTR];
+    SAFECPY(name, base_name_sans(path, ".nbr"));
+    return name;
+}
+
+/*---------------------------------------------------------------------------*/
+
 static int demo_header_read(fs_file fp, struct demo *d)
 {
     int magic;
     int version;
-    int t, unused;
+    int t;
 
     struct tm date;
     char datestr[DATELEN];
 
-    get_index(fp, &magic);
-    get_index(fp, &version);
+    magic   = get_index(fp);
+    version = get_index(fp);
 
-    get_index(fp, &t);
+    t = get_index(fp);
 
     if (magic == DEMO_MAGIC && version == DEMO_VERSION && t)
     {
         d->timer = t;
 
-        get_index(fp, &d->coins);
-        get_index(fp, &d->status);
-        get_index(fp, &d->mode);
+        d->coins  = get_index(fp);
+        d->status = get_index(fp);
+        d->mode   = get_index(fp);
 
         get_string(fp, d->player, sizeof (d->player));
         get_string(fp, datestr, sizeof (datestr));
@@ -87,12 +100,12 @@ static int demo_header_read(fs_file fp, struct demo *d)
         get_string(fp, d->shot, PATHMAX);
         get_string(fp, d->file, PATHMAX);
 
-        get_index(fp, &d->time);
-        get_index(fp, &d->goal);
-        get_index(fp, &unused);
-        get_index(fp, &d->score);
-        get_index(fp, &d->balls);
-        get_index(fp, &d->times);
+        d->time  = get_index(fp);
+        d->goal  = get_index(fp);
+        (void)     get_index(fp);
+        d->score = get_index(fp);
+        d->balls = get_index(fp);
+        d->times = get_index(fp);
 
         return 1;
     }
@@ -128,46 +141,33 @@ static void demo_header_write(fs_file fp, struct demo *d)
 
 /*---------------------------------------------------------------------------*/
 
-struct demo *demo_load(const char *path)
+int demo_load(struct demo *d, const char *path)
 {
-    fs_file fp;
-    struct demo *d;
+    int rc = 0;
 
-    d = NULL;
-
-    if ((fp = fs_open(path, "r")))
+    if (d)
     {
-        d = calloc(1, sizeof (struct demo));
+        fs_file fp;
 
-        if (demo_header_read(fp, d))
-        {
-            SAFECPY(d->filename, path);
-            SAFECPY(d->name, base_name_sans(d->filename, ".nbr"));
-        }
-        else
-        {
-            free(d);
-            d = NULL;
-        }
+        memset(d, 0, sizeof (*d));
 
-        fs_close(fp);
+        if ((fp = fs_open(path, "r")))
+        {
+            SAFECPY(d->path, path);
+            SAFECPY(d->name, demo_name(path));
+
+            if (demo_header_read(fp, d))
+                rc = 1;
+
+            fs_close(fp);
+        }
     }
 
-    return d;
+    return rc;
 }
 
 void demo_free(struct demo *d)
 {
-    free(d);
-}
-
-/*---------------------------------------------------------------------------*/
-
-static const char *demo_path(const char *name)
-{
-    static char path[MAXSTR];
-    sprintf(path, "Replays/%s.nbr", name);
-    return path;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -224,13 +224,12 @@ const char *demo_format_name(const char *fmt,
                 break;
 
             case '\0':
-                fputs(L_("Missing format character in replay name\n"), stderr);
+                /* Missing format. */
                 fmt--;
                 break;
 
             default:
-                fprintf(stderr, L_("Invalid format character in "
-                                   "replay name: \"%%%c\"\n"), *fmt);
+                /* Invalid format. */
                 break;
             }
         }
@@ -267,29 +266,32 @@ const char *demo_format_name(const char *fmt,
 
 /*---------------------------------------------------------------------------*/
 
+static struct demo demo_play;
+
 int demo_play_init(const char *name, const struct level *level,
                    int mode, int scores, int balls, int times)
 {
-    struct demo demo;
+    struct demo *d = &demo_play;
 
-    memset(&demo, 0, sizeof (demo));
+    memset(d, 0, sizeof (*d));
 
-    SAFECPY(demo.filename, demo_path(name));
-    SAFECPY(demo.player, config_get_s(CONFIG_PLAYER));
-    SAFECPY(demo.shot, level_shot(level));
-    SAFECPY(demo.file, level_file(level));
+    SAFECPY(d->path,   demo_path(name));
+    SAFECPY(d->name,   name);
+    SAFECPY(d->player, config_get_s(CONFIG_PLAYER));
+    SAFECPY(d->shot,   level_shot(level));
+    SAFECPY(d->file,   level_file(level));
 
-    demo.mode  = mode;
-    demo.date  = time(NULL);
-    demo.time  = level_time(level);
-    demo.goal  = level_goal(level);
-    demo.score = scores;
-    demo.balls = balls;
-    demo.times = times;
+    d->mode  = mode;
+    d->date  = time(NULL);
+    d->time  = level_time(level);
+    d->goal  = level_goal(level);
+    d->score = scores;
+    d->balls = balls;
+    d->times = times;
 
-    if ((demo_fp = fs_open(demo.filename, "w")))
+    if ((demo_fp = fs_open(d->path, "w")))
     {
-        demo_header_write(demo_fp, &demo);
+        demo_header_write(demo_fp, d);
         return 1;
     }
     return 0;
@@ -311,33 +313,32 @@ void demo_play_stat(int status, int coins, int timer)
     }
 }
 
-void demo_play_stop(void)
+void demo_play_stop(int d)
 {
     if (demo_fp)
     {
         fs_close(demo_fp);
         demo_fp = NULL;
+
+        if (d) fs_remove(demo_play.path);
     }
 }
 
 int demo_saved(void)
 {
-    return demo_exists(USER_REPLAY_FILE);
+    return fs_exists(demo_play.path);
 }
 
 void demo_rename(const char *name)
 {
-    char src[MAXSTR];
-    char dst[MAXSTR];
+    char path[MAXSTR];
 
-    if (name &&
-        demo_exists(USER_REPLAY_FILE) &&
-        strcmp(name, USER_REPLAY_FILE) != 0)
+    if (name && *name)
     {
-        SAFECPY(src, demo_path(USER_REPLAY_FILE));
-        SAFECPY(dst, demo_path(name));
+        SAFECPY(path, demo_path(name));
 
-        fs_rename(src, dst);
+        if (strcmp(demo_play.name, name) != 0 && fs_exists(demo_play.path))
+            fs_rename(demo_play.path, path);
     }
 }
 
@@ -387,22 +388,21 @@ static struct demo demo_replay;
 
 const char *curr_demo(void)
 {
-    return demo_replay.filename;
+    return demo_replay.path;
 }
 
-int demo_replay_init(const char *name, int *g, int *m, int *b, int *s, int *tt)
+int demo_replay_init(const char *path, int *g, int *m, int *b, int *s, int *tt)
 {
     lockstep_clr(&update_step);
 
-    if ((demo_fp = fs_open(name, "r")))
+    if ((demo_fp = fs_open(path, "r")))
     {
         if (demo_header_read(demo_fp, &demo_replay))
         {
             struct level level;
 
-            SAFECPY(demo_replay.filename, name);
-            SAFECPY(demo_replay.name,
-                    base_name_sans(demo_replay.filename, ".nbr"));
+            SAFECPY(demo_replay.path, path);
+            SAFECPY(demo_replay.name, demo_name(path));
 
             if (level_load(demo_replay.file, &level))
             {
@@ -462,11 +462,11 @@ void demo_replay_stop(int d)
         fs_close(demo_fp);
         demo_fp = NULL;
 
-        if (d) fs_remove(demo_replay.filename);
+        if (d) fs_remove(demo_replay.path);
     }
 }
 
-void demo_speed_set(int speed)
+void demo_replay_speed(int speed)
 {
     if (SPEED_NONE <= speed && speed < SPEED_MAX)
         lockstep_scl(&update_step, SPEED_FACTORS[speed]);
