@@ -16,6 +16,7 @@
 #include "vec3.h"
 #include "config.h"
 #include "solid_vary.h"
+#include "hmd.h"
 #include "common.h"
 
 /*---------------------------------------------------------------------------*/
@@ -24,23 +25,43 @@ const char *status_to_str(int s)
 {
     switch (s)
     {
-    case GAME_NONE:    return _("Aborted");
-    case GAME_TIME:    return _("Time-out");
-    case GAME_GOAL:    return _("Success");
-    case GAME_FALL:    return _("Fall-out");
-    default:           return _("Unknown");
+    case GAME_NONE: return _("Aborted");
+    case GAME_TIME: return _("Time-out");
+    case GAME_GOAL: return _("Success");
+    case GAME_FALL: return _("Fall-out");
+    default:        return _("Unknown");
     }
 }
 
-const char *view_to_str(int v)
+/*---------------------------------------------------------------------------*/
+
+const char *cam_to_str(int c)
 {
-    switch (v)
-    {
-    case VIEW_CHASE:   return _("Chase");
-    case VIEW_LAZY:    return _("Lazy");
-    case VIEW_MANUAL:  return _("Manual");
-    default:           return _("Unknown");
-    }
+    static char str[64];
+
+    int s = cam_speed(c);
+
+    if (s <    0) return _("Manual Camera");
+    if (s <= 100) return _("Lazy Camera");
+    if (s <= 500) return _("Chase Camera");
+
+    sprintf(str, _("Camera %d"), c + 1);
+
+    return str;
+}
+
+int cam_speed(int c)
+{
+    static const int *cfgs[] = {
+        &CONFIG_CAMERA_1_SPEED,
+        &CONFIG_CAMERA_2_SPEED,
+        &CONFIG_CAMERA_3_SPEED
+    };
+
+    if (c >= 0 && c < ARRAYSIZE(cfgs))
+        return config_get_d(*cfgs[c]);
+
+    return 250;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -90,10 +111,22 @@ void game_tilt_grav(float h[3], const float g[3], const struct game_tilt *tilt)
 
 void game_view_init(struct game_view *view)
 {
-    view->dp  = config_get_d(CONFIG_VIEW_DP) / 100.0f;
-    view->dc  = config_get_d(CONFIG_VIEW_DC) / 100.0f;
-    view->dz  = config_get_d(CONFIG_VIEW_DZ) / 100.0f;
-    view->a   = 0.0f;
+    /* In VR, ensure the default view is level. */
+
+    if (hmd_stat())
+    {
+        view->dp = config_get_d(CONFIG_VIEW_DP) / 100.0f;
+        view->dc = config_get_d(CONFIG_VIEW_DP) / 100.0f;
+        view->dz = config_get_d(CONFIG_VIEW_DZ) / 100.0f;
+    }
+    else
+    {
+        view->dp = config_get_d(CONFIG_VIEW_DP) / 100.0f;
+        view->dc = config_get_d(CONFIG_VIEW_DC) / 100.0f;
+        view->dz = config_get_d(CONFIG_VIEW_DZ) / 100.0f;
+    }
+
+    view->a = 0.0f;
 
     view->c[0] = 0.0f;
     view->c[1] = view->dc;
@@ -241,3 +274,16 @@ void game_base_free(const char *next)
 }
 
 /*---------------------------------------------------------------------------*/
+
+float SPEED_FACTORS[SPEED_MAX] = {
+    0.0f,
+    1.0f / 8,
+    1.0f / 4,
+    1.0f / 2,
+    1.0f,
+    1.0f * 2,
+    1.0f * 4,
+    1.0f * 8
+};
+
+/*---------------------------------------------------------------------------*/
