@@ -47,16 +47,18 @@ const char *status_to_str(int);
 
 enum
 {
-    VIEW_NONE = -1,
+    CAM_NONE = -1,
 
-    VIEW_CHASE,
-    VIEW_LAZY,
-    VIEW_MANUAL,
+    CAM_1,
+    CAM_2,
+    CAM_3,
 
-    VIEW_MAX
+    CAM_MAX
 };
 
-const char *view_to_str(int);
+const char *cam_to_str(int);
+
+int cam_speed(int);
 
 /*---------------------------------------------------------------------------*/
 
@@ -93,7 +95,7 @@ void game_view_fly(struct game_view *, const struct s_vary *, float);
 
 /*---------------------------------------------------------------------------*/
 
-#define UPS 30
+#define UPS 90
 #define DT  (1.0f / (float) UPS)
 
 /*
@@ -124,4 +126,26 @@ void game_base_free(const char *);
 
 /*---------------------------------------------------------------------------*/
 
+enum
+{
+    SPEED_NONE = 0,
+
+    SPEED_SLOWEST,
+    SPEED_SLOWER,
+    SPEED_SLOW,
+    SPEED_NORMAL,
+    SPEED_FAST,
+    SPEED_FASTER,
+    SPEED_FASTEST,
+
+    SPEED_MAX
+};
+
+extern float SPEED_FACTORS[];
+
+#define SPEED_UP(s) MIN((s) + 1, SPEED_MAX - 1)
+#define SPEED_DN(s) MAX((s) - 1, SPEED_NONE)
+
+/*---------------------------------------------------------------------------*/
+
 #endif
