@@ -1,9 +1,6 @@
 #ifndef GAME_SERVER_H
 #define GAME_SERVER_H
 
-#include "solid.h"
-#include "fs.h"
-
 /*---------------------------------------------------------------------------*/
 
 #define RESPONSE    0.05f              /* Input smoothing time               */
@@ -13,24 +10,18 @@
 /*---------------------------------------------------------------------------*/
 
 int   game_server_init(const char *, int, int);
-void  game_server_free(void);
+void  game_server_free(const char *);
 void  game_server_step(float);
+float game_server_blend(void);
 
 void  game_set_goal(void);
-void  game_clr_goal(void);
 
-void  game_set_ang(int, int);
+void  game_set_ang(float, float);
 void  game_set_pos(int, int);
-void  game_set_x  (int);
-void  game_set_z  (int);
+void  game_set_x  (float);
+void  game_set_z  (float);
 void  game_set_cam(int);
 void  game_set_rot(float);
-void  game_set_fly(float, const struct s_file *);
-
-/*---------------------------------------------------------------------------*/
-
-int input_put(fs_file);
-int input_get(fs_file);
 
 /*---------------------------------------------------------------------------*/
 
