@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "version.h"
 #include "glext.h"
 #include "config.h"
 #include "video.h"
@@ -28,11 +29,12 @@
 #include "gui.h"
 #include "set.h"
 #include "tilt.h"
+#include "hmd.h"
 #include "fs.h"
 #include "common.h"
-#ifdef __MAEMO__
-#include "maemo.h"
-#endif
+#include "text.h"
+#include "mtrl.h"
+#include "geom.h"
 
 #include "st_conf.h"
 #include "st_title.h"
@@ -45,23 +47,11 @@ const char ICON[] = "icon/neverball.png";
 
 /*---------------------------------------------------------------------------*/
 
-static int shot_pending;
-
-static void shot_prep(void)
-{
-    shot_pending = 1;
-}
-
-static void shot_take(void)
+static void shot(void)
 {
     static char filename[MAXSTR];
-
-    if (shot_pending)
-    {
-        sprintf(filename, "Screenshots/screen%05d.png", config_screenshot());
-        image_snap(filename);
-        shot_pending = 0;
-    }
+    sprintf(filename, "Screenshots/screen%05d.png", config_screenshot());
+    video_snap(filename);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -74,15 +64,11 @@ static void toggle_wire(void)
     if (wire)
     {
         glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-        glEnable(GL_TEXTURE_2D);
-        glEnable(GL_LIGHTING);
         wire = 0;
     }
     else
     {
         glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
-        glDisable(GL_TEXTURE_2D);
-        glDisable(GL_LIGHTING);
         wire = 1;
     }
 #endif
@@ -93,42 +79,53 @@ static void toggle_wire(void)
 static int handle_key_dn(SDL_Event *e)
 {
     int d = 1;
-    int c;
+    int c = e->key.keysym.sym;
 
-    c = e->key.keysym.sym;
+    /* SDL made me do it. */
+#ifdef __APPLE__
+    if (c == SDLK_q && e->key.keysym.mod & KMOD_GUI)
+        return 0;
+#endif
+#ifdef _WIN32
+    if (c == SDLK_F4 && e->key.keysym.mod & KMOD_ALT)
+        return 0;
+#endif
 
-    if (config_tst_d(CONFIG_KEY_FORWARD, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y), -1.0f);
-
-    else if (config_tst_d(CONFIG_KEY_BACKWARD, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y), +1.0f);
-
-    else if (config_tst_d(CONFIG_KEY_LEFT, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X), -1.0f);
-
-    else if (config_tst_d(CONFIG_KEY_RIGHT, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X), +1.0f);
-
-    else switch (c)
+    switch (c)
     {
-    case SDLK_F10:   shot_prep();               break;
-    case SDLK_F9:    config_tgl_d(CONFIG_FPS);  break;
-    case SDLK_F8:    config_tgl_d(CONFIG_NICE); break;
-
-    case SDLK_F7:
+    case KEY_SCREENSHOT:
+        shot();
+        break;
+    case KEY_FPS:
+        config_tgl_d(CONFIG_FPS);
+        break;
+    case KEY_WIREFRAME:
         if (config_cheat())
             toggle_wire();
         break;
+    case KEY_RESOURCES:
+        if (config_cheat())
+        {
+            light_load();
+            mtrl_reload();
+        }
+        break;
     case SDLK_RETURN:
         d = st_buttn(config_get_d(CONFIG_JOYSTICK_BUTTON_A), 1);
         break;
-    case SDLK_ESCAPE:
-        d = st_buttn(config_get_d(CONFIG_JOYSTICK_BUTTON_EXIT), 1);
+    case KEY_EXIT:
+        d = st_keybd(KEY_EXIT, 1);
         break;
 
     default:
-        if (SDL_EnableUNICODE(-1))
-            d = st_keybd(e->key.keysym.unicode, 1);
+        if (config_tst_d(CONFIG_KEY_FORWARD, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y0), -1.0f);
+        else if (config_tst_d(CONFIG_KEY_BACKWARD, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y0), +1.0f);
+        else if (config_tst_d(CONFIG_KEY_LEFT, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X0), -1.0f);
+        else if (config_tst_d(CONFIG_KEY_RIGHT, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X0), +1.0f);
         else
             d = st_keybd(e->key.keysym.sym, 1);
     }
@@ -139,33 +136,27 @@ static int handle_key_dn(SDL_Event *e)
 static int handle_key_up(SDL_Event *e)
 {
     int d = 1;
-    int c;
+    int c = e->key.keysym.sym;
 
-    c = e->key.keysym.sym;
-
-    if (config_tst_d(CONFIG_KEY_FORWARD, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y), 0);
-
-    else if (config_tst_d(CONFIG_KEY_BACKWARD, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y), 0);
-
-    else if (config_tst_d(CONFIG_KEY_LEFT, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X), 0);
-
-    else if (config_tst_d(CONFIG_KEY_RIGHT, c))
-        st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X), 0);
-
-    else switch (c)
+    switch (c)
     {
     case SDLK_RETURN:
         d = st_buttn(config_get_d(CONFIG_JOYSTICK_BUTTON_A), 0);
         break;
-    case SDLK_ESCAPE:
-        d = st_buttn(config_get_d(CONFIG_JOYSTICK_BUTTON_EXIT), 0);
+    case KEY_EXIT:
+        d = st_keybd(KEY_EXIT, 0);
         break;
-
     default:
-        d = st_keybd(e->key.keysym.sym, 0);
+        if (config_tst_d(CONFIG_KEY_FORWARD, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y0), 0);
+        else if (config_tst_d(CONFIG_KEY_BACKWARD, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_Y0), 0);
+        else if (config_tst_d(CONFIG_KEY_LEFT, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X0), 0);
+        else if (config_tst_d(CONFIG_KEY_RIGHT, c))
+            st_stick(config_get_d(CONFIG_JOYSTICK_AXIS_X0), 0);
+        else
+            d = st_keybd(e->key.keysym.sym, 0);
     }
 
     return d;
@@ -176,6 +167,8 @@ static int loop(void)
     SDL_Event e;
     int d = 1;
 
+    int ax, ay, dx, dy;
+
     /* Process SDL events. */
 
     while (d && SDL_PollEvent(&e))
@@ -186,11 +179,23 @@ static int loop(void)
             return 0;
 
         case SDL_MOUSEMOTION:
-            st_point(+e.motion.x,
-                     -e.motion.y + config_get_d(CONFIG_HEIGHT),
-                     +e.motion.xrel,
-                     config_get_d(CONFIG_MOUSE_INVERT)
-                     ? +e.motion.yrel : -e.motion.yrel);
+            /* Convert to OpenGL coordinates. */
+
+            ax = +e.motion.x;
+            ay = -e.motion.y + video.window_h;
+            dx = +e.motion.xrel;
+            dy = (config_get_d(CONFIG_MOUSE_INVERT) ?
+                  +e.motion.yrel : -e.motion.yrel);
+
+            /* Convert to pixels. */
+
+            ax = ROUND(ax * video.device_scale);
+            ay = ROUND(ay * video.device_scale);
+            dx = ROUND(dx * video.device_scale);
+            dy = ROUND(dy * video.device_scale);
+
+            st_point(ax, ay, dx, dy);
+
             break;
 
         case SDL_MOUSEBUTTONDOWN:
@@ -209,10 +214,37 @@ static int loop(void)
             d = handle_key_up(&e);
             break;
 
-        case SDL_ACTIVEEVENT:
-            if (e.active.state == SDL_APPINPUTFOCUS)
-                if (e.active.gain == 0 && video_get_grab())
+        case SDL_WINDOWEVENT:
+            switch (e.window.event)
+            {
+            case SDL_WINDOWEVENT_FOCUS_LOST:
+                if (video_get_grab())
                     goto_state(&st_pause);
+                break;
+
+            case SDL_WINDOWEVENT_MOVED:
+                if (config_get_d(CONFIG_DISPLAY) != video_display())
+                    config_set_d(CONFIG_DISPLAY, video_display());
+                break;
+
+            case SDL_WINDOWEVENT_RESIZED:
+                log_printf("Resize event (%u, %dx%d)\n",
+                           e.window.windowID,
+                           e.window.data1,
+                           e.window.data2);
+                break;
+
+            case SDL_WINDOWEVENT_SIZE_CHANGED:
+                log_printf("Size change event (%u, %dx%d)\n",
+                           e.window.windowID,
+                           e.window.data1,
+                           e.window.data2);
+                break;
+            }
+            break;
+
+        case SDL_TEXTINPUT:
+            text_input_str(e.text.text, 1);
             break;
 
         case SDL_JOYAXISMOTION:
@@ -226,6 +258,10 @@ static int loop(void)
         case SDL_JOYBUTTONUP:
             d = st_buttn(e.jbutton.button, 0);
             break;
+
+        case SDL_MOUSEWHEEL:
+            st_wheel(e.wheel.x, e.wheel.y);
+            break;
         }
     }
 
@@ -241,8 +277,8 @@ static int loop(void)
 
         while (tilt_get_button(&b, &s))
         {
-            const int X = config_get_d(CONFIG_JOYSTICK_AXIS_X);
-            const int Y = config_get_d(CONFIG_JOYSTICK_AXIS_Y);
+            const int X = config_get_d(CONFIG_JOYSTICK_AXIS_X0);
+            const int Y = config_get_d(CONFIG_JOYSTICK_AXIS_Y0);
             const int L = config_get_d(CONFIG_JOYSTICK_DPAD_L);
             const int R = config_get_d(CONFIG_JOYSTICK_DPAD_R);
             const int U = config_get_d(CONFIG_JOYSTICK_DPAD_U);
@@ -282,19 +318,17 @@ static char *opt_data;
 static char *opt_replay;
 static char *opt_level;
 
-#define opt_usage \
-    L_(                                                                   \
-        "Usage: %s [options ...]\n"                                       \
-        "Options:\n"                                                      \
-        "  -h, --help                show this usage message.\n"          \
-        "  -v, --version             show version.\n"                     \
-        "  -d, --data <dir>          use 'dir' as game data directory.\n" \
-        "  -r, --replay <file>       play the replay 'file'.\n"           \
-        "  -l, --level <file>        load the level 'file'\n"             \
-    )
+#define opt_usage                                                     \
+    "Usage: %s [options ...]\n"                                       \
+    "Options:\n"                                                      \
+    "  -h, --help                show this usage message.\n"          \
+    "  -v, --version             show version.\n"                     \
+    "  -d, --data <dir>          use 'dir' as game data directory.\n" \
+    "  -r, --replay <file>       play the replay 'file'.\n"           \
+    "  -l, --level <file>        load the level 'file'\n"
 
 #define opt_error(option) \
-    fprintf(stderr, L_("Option '%s' requires an argument.\n"), option)
+    fprintf(stderr, "Option '%s' requires an argument.\n", option)
 
 static void opt_parse(int argc, char **argv)
 {
@@ -447,28 +481,22 @@ int main(int argc, char *argv[])
 
     if (!fs_init(argv[0]))
     {
-        fprintf(stderr, "Failure to initialize virtual file system: %s\n",
+        fprintf(stderr, "Failure to initialize virtual file system (%s)\n",
                 fs_error());
         return 1;
     }
 
-    lang_init("neverball");
-
     opt_parse(argc, argv);
 
     config_paths(opt_data);
+    log_init("Neverball", "neverball.log");
     make_dirs_and_migrate();
 
     /* Initialize SDL. */
 
-    if (SDL_Init(SDL_INIT_VIDEO |
-                 SDL_INIT_AUDIO |
-#ifdef __MAEMO__
-                 SDL_INIT_TIMER |
-#endif
-                 SDL_INIT_JOYSTICK) == -1)
+    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) == -1)
     {
-        fprintf(stderr, "%s\n", SDL_GetError());
+        log_printf("Failure to initialize SDL (%s)\n", SDL_GetError());
         return 1;
     }
 
@@ -477,6 +505,10 @@ int main(int argc, char *argv[])
     config_init();
     config_load();
 
+    /* Initialize localization. */
+
+    lang_init();
+
     /* Initialize joystick. */
 
     if (config_get_d(CONFIG_JOYSTICK) && SDL_NumJoysticks() > 0)
@@ -493,9 +525,15 @@ int main(int argc, char *argv[])
 
     /* Initialize video. */
 
-    if (!video_init(TITLE, ICON))
+    if (!video_init())
         return 1;
 
+    /* Material system. */
+
+    mtrl_init();
+
+    /* Screen states. */
+
     init_state(&st_null);
 
     /* Initialize demo playback or load the level. */
@@ -528,7 +566,7 @@ int main(int argc, char *argv[])
                 }
             }
         }
-        else fprintf(stderr, "%s: file is not in game path\n", opt_level);
+        else log_printf("File %s is not in game path\n", opt_level);
 
         if (!loaded)
             goto_state(&st_title);
@@ -536,10 +574,6 @@ int main(int argc, char *argv[])
     else
         goto_state(&st_title);
 
-#ifdef __MAEMO__
-    maemo_init("neverball");
-#endif
-
     /* Run the main game loop. */
 
     t0 = SDL_GetTicks();
@@ -556,8 +590,8 @@ int main(int argc, char *argv[])
 
             /* Render. */
 
+            hmd_step();
             st_paint(0.001f * t0);
-            shot_take();
             video_swap();
 
             if (config_get_d(CONFIG_NICE))
@@ -567,13 +601,13 @@ int main(int argc, char *argv[])
 
     config_save();
 
+    mtrl_quit();
+
     if (joy)
         SDL_JoystickClose(joy);
 
     tilt_free();
-#ifdef __MAEMO__
-    maemo_quit();
-#endif
+    hmd_free();
     SDL_Quit();
 
     return 0;
