@@ -35,13 +35,6 @@
 #include "st_conf.h"
 #include "st_shared.h"
 
-enum
-{
-    BALL_NEXT = 1,
-    BALL_PREV,
-    BALL_BACK
-};
-
 static Array balls;
 static int   curr_ball;
 static char  ball_file[64];
@@ -78,6 +71,12 @@ static int has_ball_sols(struct dir_item *item)
     return yes;
 }
 
+static int cmp_dir_items(const void *A, const void *B)
+{
+    const struct dir_item *a = A, *b = B;
+    return strcmp(a->path, b->path);
+}
+
 static void scan_balls(void)
 {
     int i;
@@ -86,6 +85,8 @@ static void scan_balls(void)
 
     if ((balls = fs_dir_scan("ball", has_ball_sols)))
     {
+        array_sort(balls, cmp_dir_items);
+
         for (i = 0; i < array_len(balls); i++)
         {
             const char *path = DIR_ITEM_GET(balls, i)->path;
@@ -119,13 +120,13 @@ static void set_curr_ball(void)
     gui_set_label(name_id, base_name(ball_file));
 }
 
-static int ball_action(int i)
+static int ball_action(int tok, int val)
 {
     audio_play(AUD_MENU, 1.0f);
 
-    switch (i)
+    switch (tok)
     {
-    case BALL_NEXT:
+    case GUI_NEXT:
         if (++curr_ball == array_len(balls))
             curr_ball = 0;
 
@@ -133,7 +134,7 @@ static int ball_action(int i)
 
         break;
 
-    case BALL_PREV:
+    case GUI_PREV:
         if (--curr_ball == -1)
             curr_ball = array_len(balls) - 1;
 
@@ -141,7 +142,7 @@ static int ball_action(int i)
 
         break;
 
-    case BALL_BACK:
+    case GUI_BACK:
         goto_state(&st_conf);
         break;
     }
@@ -155,11 +156,15 @@ static void load_ball_demo(void)
 
     /* "g" is a stupid hack to keep the goal locked. */
 
-    demo_replay_init("gui/ball.nbr", &g, NULL, NULL, NULL, NULL);
+    if (!demo_replay_init("gui/ball.nbr", &g, NULL, NULL, NULL, NULL))
+    {
+        ball_action(GUI_BACK, 0);
+        return;
+    }
+
     audio_music_fade_to(0.0f, "bgm/inter.ogg");
     game_client_fly(0);
     game_kill_fade();
-
     back_init("back/gui.png");
 }
 
@@ -172,25 +177,24 @@ static int ball_gui(void)
     {
         if ((jd = gui_harray(id)))
         {
-            gui_label(jd, _("Ball Model"), GUI_SML, GUI_ALL, 0, 0);
+            gui_label(jd, _("Ball Model"), GUI_SML, 0, 0);
             gui_space(jd);
-            gui_start(jd, _("Back"), GUI_SML, BALL_BACK, 0);
+            gui_start(jd, _("Back"), GUI_SML, GUI_BACK, 0);
         }
 
         gui_space(id);
 
         if ((jd = gui_hstack(id)))
         {
-            gui_state(jd, " > ", GUI_SML, BALL_NEXT, 0);
+            gui_state(jd, " > ", GUI_SML, GUI_NEXT, 0);
 
-            name_id = gui_label(jd, "very-long-ball-name",
-                                GUI_SML, GUI_ALL,
+            name_id = gui_label(jd, "very-long-ball-name", GUI_SML,
                                 gui_wht, gui_wht);
 
             gui_set_trunc(name_id, TRUNC_TAIL);
             gui_set_fill(name_id);
 
-            gui_state(jd, " < ", GUI_SML, BALL_PREV, 0);
+            gui_state(jd, " < ", GUI_SML, GUI_PREV, 0);
         }
 
         for (i = 0; i < 12; i++)
@@ -245,15 +249,27 @@ static void ball_timer(int id, float dt)
     game_client_blend(demo_replay_blend());
 }
 
+static int ball_keybd(int c, int d)
+{
+    if (d)
+    {
+        if (c == KEY_EXIT)
+            return ball_action(GUI_BACK, 0);
+    }
+    return 1;
+}
+
 static int ball_buttn(int b, int d)
 {
     if (d)
     {
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
-            return ball_action(gui_token(gui_click()));
+        int active = gui_active();
 
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return ball_action(BALL_BACK);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
+            return ball_action(gui_token(active), gui_value(active));
+
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            return ball_action(GUI_BACK, 0);
     }
     return 1;
 }
@@ -267,6 +283,6 @@ struct state st_ball = {
     shared_stick,
     NULL,
     shared_click,
-    NULL,
+    ball_keybd,
     ball_buttn
 };
