@@ -65,18 +65,20 @@ static int level_gui(void)
                 else
                     sprintf(setattr, "%s", set_name(curr_set()));
 
-                gui_label(kd, lvlattr, b ? GUI_MED : GUI_LRG, GUI_TOP,
-                          b ? gui_wht : 0, b ? gui_grn : 0);
-                gui_label(kd, setattr, GUI_SML,               GUI_BOT,
-                          gui_wht,         gui_wht);
+                gui_label(kd, lvlattr,
+                          b ? GUI_MED : GUI_LRG,
+                          b ? gui_wht : 0,
+                          b ? gui_grn : 0);
+
+                gui_label(kd, setattr, GUI_SML, gui_wht, gui_wht);
+
+                gui_set_rect(kd, GUI_ALL);
             }
             gui_filler(jd);
         }
         gui_space(id);
 
-        gui_multi(id, level_msg(curr_level()),
-                  GUI_SML, GUI_ALL,
-                  gui_wht, gui_wht);
+        gui_multi(id, level_msg(curr_level()), GUI_SML, gui_wht, gui_wht);
 
         gui_layout(id, 0, 0);
     }
@@ -110,8 +112,16 @@ static int level_click(int b, int d)
 
 static int level_keybd(int c, int d)
 {
-    if (d && c == SDLK_F12)
-        return goto_state(&st_poser);
+    if (d)
+    {
+        if (c == KEY_EXIT)
+        {
+            progress_stop();
+            return goto_state(&st_exit);
+        }
+        if (c == KEY_POSE)
+            return goto_state(&st_poser);
+    }
     return 1;
 }
 
@@ -119,11 +129,12 @@ static int level_buttn(int b, int d)
 {
     if (d)
     {
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b) ||
+            config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
         {
             return goto_state(&st_play_ready);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
         {
             progress_stop();
             return goto_state(&st_exit);
@@ -139,9 +150,19 @@ static void poser_paint(int id, float t)
     game_client_draw(POSE_LEVEL, t);
 }
 
+static int poser_keybd(int c, int d)
+{
+    if (d)
+    {
+        if (c == KEY_EXIT || c == KEY_POSE)
+            return goto_state(&st_level);
+    }
+    return 1;
+}
+
 static int poser_buttn(int c, int d)
 {
-    if (d && config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, c))
+    if (d && config_tst_d(CONFIG_JOYSTICK_BUTTON_B, c))
         return goto_state(&st_level);
 
     return 1;
@@ -155,12 +176,11 @@ static int nodemo_gui(void)
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, _("Warning!"), GUI_MED, GUI_ALL, 0, 0);
+        gui_label(id, _("Warning!"), GUI_MED, 0, 0);
         gui_space(id);
-        gui_multi(id, _("The default replay file could not be opened.\\"
-                        "You will not be able to save a replay when\\"
-                        "playing this level.\\") ,
-                  GUI_SML, GUI_ALL, gui_wht, gui_wht);
+        gui_multi(id, _("A replay file could not be opened for writing.\\"
+                        "This game will not be recorded.\\"),
+                  GUI_SML, gui_wht, gui_wht);
 
         gui_layout(id, 0, 0);
     }
@@ -181,13 +201,23 @@ static void nodemo_timer(int id, float dt)
     gui_timer(id, dt);
 }
 
+static int nodemo_keybd(int c, int d)
+{
+    if (d)
+    {
+        if (c == KEY_EXIT)
+            return goto_state(&st_level);
+    }
+    return 1;
+}
+
 static int nodemo_buttn(int b, int d)
 {
     if (d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_level);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return goto_state(&st_level);
     }
     return 1;
@@ -223,7 +253,9 @@ static int exit_enter(struct state *st, struct state *prev)
         SDL_PushEvent(&e);
     }
 
-    return 0;
+    /* HACK: The GUI ID gets lost if you goto_state during a goto_state. */
+
+    return curr_state()->gui_id;
 }
 
 /*---------------------------------------------------------------------------*/
@@ -250,7 +282,7 @@ struct state st_poser = {
     NULL,
     NULL,
     NULL,
-    NULL,
+    poser_keybd,
     poser_buttn
 };
 
@@ -262,8 +294,8 @@ struct state st_nodemo = {
     shared_point,
     shared_stick,
     shared_angle,
-    shared_click,
-    NULL,
+    shared_click_basic,
+    nodemo_keybd,
     nodemo_buttn
 };
 
