@@ -33,17 +33,13 @@
 
 /*---------------------------------------------------------------------------*/
 
-static char player[MAXNAM];
+static struct state *ok_state;
+static struct state *cancel_state;
 
-/*---------------------------------------------------------------------------*/
-
-static struct state *ok_state, *cancel_state;
 static unsigned int draw_back;
 
 int goto_name(struct state *ok, struct state *cancel, unsigned int back)
 {
-    SAFECPY(player, config_get_s(CONFIG_PLAYER));
-
     ok_state     = ok;
     cancel_state = cancel;
     draw_back    = back;
@@ -53,40 +49,41 @@ int goto_name(struct state *ok, struct state *cancel, unsigned int back)
 
 /*---------------------------------------------------------------------------*/
 
-#define NAME_OK     -1
-#define NAME_CANCEL -2
+enum
+{
+    NAME_OK = GUI_LAST
+};
 
 static int name_id;
 
-static int name_action(int i)
+static int name_action(int tok, int val)
 {
     audio_play(AUD_MENU, 1.0f);
 
-    switch (i)
+    switch (tok)
     {
+    case GUI_BACK:
+        return goto_state(cancel_state);
+
     case NAME_OK:
-        if (strlen(player) == 0)
+        if (strlen(text_input) == 0)
            return 1;
 
-        config_set_s(CONFIG_PLAYER, player);
+        config_set_s(CONFIG_PLAYER, text_input);
 
         return goto_state(ok_state);
 
-    case NAME_CANCEL:
-        return goto_state(cancel_state);
-
     case GUI_CL:
         gui_keyboard_lock();
         break;
 
     case GUI_BS:
-        if (text_del_char(player))
-            gui_set_label(name_id, player);
+        text_input_del();
         break;
 
-    default:
-        if (text_add_char(i, player, sizeof (player)))
-            gui_set_label(name_id, player);
+    case GUI_CHAR:
+        text_input_char(val);
+        break;
     }
     return 1;
 }
@@ -99,10 +96,10 @@ static int name_gui(void)
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, _("Player Name"), GUI_MED, GUI_ALL, 0, 0);
+        gui_label(id, _("Player Name"), GUI_MED, 0, 0);
         gui_space(id);
 
-        name_id = gui_label(id, " ", GUI_MED, GUI_ALL, gui_yel, gui_yel);
+        name_id = gui_label(id, " ", GUI_MED, gui_yel, gui_yel);
 
         gui_space(id);
         gui_keyboard(id);
@@ -112,18 +109,29 @@ static int name_gui(void)
         {
             enter_id = gui_start(jd, _("OK"), GUI_SML, NAME_OK, 0);
             gui_space(jd);
-            gui_state(jd, _("Cancel"), GUI_SML, NAME_CANCEL, 0);
+            gui_state(jd, _("Cancel"), GUI_SML, GUI_BACK, 0);
         }
 
         gui_layout(id, 0, 0);
 
         gui_set_trunc(name_id, TRUNC_HEAD);
-        gui_set_label(name_id, player);
+        gui_set_label(name_id, text_input);
     }
 
     return id;
 }
 
+static void on_text_input(int typing)
+{
+    if (name_id)
+    {
+        gui_set_label(name_id, text_input);
+
+        if (typing)
+            audio_play(AUD_MENU, 1.0f);
+    }
+}
+
 static int name_enter(struct state *st, struct state *prev)
 {
     if (draw_back)
@@ -132,7 +140,8 @@ static int name_enter(struct state *st, struct state *prev)
         back_init("back/gui.png");
     }
 
-    SDL_EnableUNICODE(1);
+    text_input_start(on_text_input);
+    text_input_str(config_get_s(CONFIG_PLAYER), 0);
 
     return name_gui();
 }
@@ -142,7 +151,8 @@ static void name_leave(struct state *st, struct state *next, int id)
     if (draw_back)
         back_free();
 
-    SDL_EnableUNICODE(0);
+    text_input_stop();
+
     gui_delete(id);
 }
 
@@ -166,12 +176,19 @@ static int name_keybd(int c, int d)
 {
     if (d)
     {
-        gui_focus(enter_id);
+        if (c == KEY_EXIT)
+            return name_action(GUI_BACK, 0);
 
         if (c == '\b' || c == 0x7F)
-            return name_action(GUI_BS);
-        if (c >= ' ')
-            return name_action(c);
+        {
+            gui_focus(enter_id);
+            return name_action(GUI_BS, 0);
+        }
+        else
+        {
+            gui_focus(enter_id);
+            return 1;
+        }
     }
     return 1;
 }
@@ -182,15 +199,15 @@ static int name_buttn(int b, int d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
         {
-            int c = gui_token(gui_click());
+            int tok = gui_token(gui_active());
+            int val = gui_value(gui_active());
 
-            if (c >= 0 && !GUI_ISMSK(c))
-                return name_action(gui_keyboard_char(c));
-            else
-                return name_action(c);
+            return name_action(tok, (tok == GUI_CHAR ?
+                                     gui_keyboard_char(val) :
+                                     val));
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            name_action(NAME_CANCEL);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            name_action(GUI_BACK, 0);
     }
     return 1;
 }
