@@ -35,35 +35,26 @@
 
 /*---------------------------------------------------------------------------*/
 
-static int pause_or_exit(void)
-{
-    if (config_tst_d(CONFIG_KEY_PAUSE, SDLK_ESCAPE))
-    {
-        return goto_state(&st_pause);
-    }
-    else
-    {
-        progress_stat(GAME_NONE);
-        progress_stop();
-
-        video_clr_grab();
-
-        return goto_state(&st_exit);
-    }
-}
-
-/*---------------------------------------------------------------------------*/
-
 static void set_camera(int c)
 {
     config_set_d(CONFIG_CAMERA, c);
-    hud_view_pulse(c);
+    hud_cam_pulse(c);
 }
 
 static void toggle_camera(void)
 {
-    int cam = (config_tst_d(CONFIG_CAMERA, VIEW_MANUAL) ?
-               VIEW_CHASE : VIEW_MANUAL);
+    int cam = (config_tst_d(CONFIG_CAMERA, CAM_3) ?
+               CAM_1 : CAM_3);
+
+    set_camera(cam);
+}
+
+static void next_camera(void)
+{
+    int cam = config_get_d(CONFIG_CAMERA) + 1;
+
+    if (cam <= CAM_NONE || cam >= CAM_MAX)
+        cam = CAM_1;
 
     set_camera(cam);
 }
@@ -71,11 +62,11 @@ static void toggle_camera(void)
 static void keybd_camera(int c)
 {
     if (config_tst_d(CONFIG_KEY_CAMERA_1, c))
-        set_camera(VIEW_CHASE);
+        set_camera(CAM_1);
     if (config_tst_d(CONFIG_KEY_CAMERA_2, c))
-        set_camera(VIEW_LAZY);
+        set_camera(CAM_2);
     if (config_tst_d(CONFIG_KEY_CAMERA_3, c))
-        set_camera(VIEW_MANUAL);
+        set_camera(CAM_3);
 
     if (config_tst_d(CONFIG_KEY_CAMERA_TOGGLE, c))
         toggle_camera();
@@ -84,11 +75,11 @@ static void keybd_camera(int c)
 static void click_camera(int b)
 {
     if (config_tst_d(CONFIG_MOUSE_CAMERA_1, b))
-        set_camera(VIEW_CHASE);
+        set_camera(CAM_1);
     if (config_tst_d(CONFIG_MOUSE_CAMERA_2, b))
-        set_camera(VIEW_LAZY);
+        set_camera(CAM_2);
     if (config_tst_d(CONFIG_MOUSE_CAMERA_3, b))
-        set_camera(VIEW_MANUAL);
+        set_camera(CAM_3);
 
     if (config_tst_d(CONFIG_MOUSE_CAMERA_TOGGLE, b))
         toggle_camera();
@@ -96,15 +87,8 @@ static void click_camera(int b)
 
 static void buttn_camera(int b)
 {
-    if (config_tst_d(CONFIG_JOYSTICK_CAMERA_1, b))
-        set_camera(VIEW_CHASE);
-    if (config_tst_d(CONFIG_JOYSTICK_CAMERA_2, b))
-        set_camera(VIEW_LAZY);
-    if (config_tst_d(CONFIG_JOYSTICK_CAMERA_3, b))
-        set_camera(VIEW_MANUAL);
-
-    if (config_tst_d(CONFIG_JOYSTICK_CAMERA_TOGGLE, b))
-        toggle_camera();
+    if (config_tst_d(CONFIG_JOYSTICK_BUTTON_X, b))
+        next_camera();
 }
 
 /*---------------------------------------------------------------------------*/
@@ -113,7 +97,7 @@ static int play_ready_gui(void)
 {
     int id;
 
-    if ((id = gui_label(0, _("Ready?"), GUI_LRG, GUI_ALL, 0, 0)))
+    if ((id = gui_label(0, _("Ready?"), GUI_LRG, 0, 0)))
     {
         gui_layout(id, 0, 0);
         gui_pulse(id, 1.2f);
@@ -127,7 +111,7 @@ static int play_ready_enter(struct state *st, struct state *prev)
     audio_play(AUD_READY, 1.0f);
     video_set_grab(1);
 
-    hud_view_pulse(config_get_d(CONFIG_CAMERA));
+    hud_cam_pulse(config_get_d(CONFIG_CAMERA));
 
     return play_ready_gui();
 }
@@ -135,7 +119,7 @@ static int play_ready_enter(struct state *st, struct state *prev)
 static void play_ready_paint(int id, float t)
 {
     game_client_draw(0, t);
-    hud_view_paint();
+    hud_cam_paint();
     gui_paint(id);
 }
 
@@ -149,7 +133,7 @@ static void play_ready_timer(int id, float dt)
         goto_state(&st_play_set);
 
     game_step_fade(dt);
-    hud_view_timer(dt);
+    hud_cam_timer(dt);
     gui_timer(id, dt);
 }
 
@@ -171,7 +155,7 @@ static int play_ready_keybd(int c, int d)
     {
         keybd_camera(c);
 
-        if (config_tst_d(CONFIG_KEY_PAUSE, c))
+        if (c == KEY_EXIT)
             goto_state(&st_pause);
     }
     return 1;
@@ -185,8 +169,8 @@ static int play_ready_buttn(int b, int d)
 
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_play_loop);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return pause_or_exit();
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_state(&st_pause);
     }
     return 1;
 }
@@ -197,7 +181,7 @@ static int play_set_gui(void)
 {
     int id;
 
-    if ((id = gui_label(0, _("Set?"), GUI_LRG, GUI_ALL, 0, 0)))
+    if ((id = gui_label(0, _("Set?"), GUI_LRG, 0, 0)))
     {
         gui_layout(id, 0, 0);
         gui_pulse(id, 1.2f);
@@ -216,7 +200,7 @@ static int play_set_enter(struct state *st, struct state *prev)
 static void play_set_paint(int id, float t)
 {
     game_client_draw(0, t);
-    hud_view_paint();
+    hud_cam_paint();
     gui_paint(id);
 }
 
@@ -230,7 +214,7 @@ static void play_set_timer(int id, float dt)
         goto_state(&st_play_loop);
 
     game_step_fade(dt);
-    hud_view_timer(dt);
+    hud_cam_timer(dt);
     gui_timer(id, dt);
 }
 
@@ -252,7 +236,7 @@ static int play_set_keybd(int c, int d)
     {
         keybd_camera(c);
 
-        if (config_tst_d(CONFIG_KEY_PAUSE, c))
+        if (c == KEY_EXIT)
             goto_state(&st_pause);
     }
     return 1;
@@ -266,35 +250,70 @@ static int play_set_buttn(int b, int d)
 
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_play_loop);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return pause_or_exit();
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_state(&st_pause);
     }
     return 1;
 }
 
 /*---------------------------------------------------------------------------*/
 
-struct
+enum
 {
-    float R;
-    float L;
+    ROT_NONE = 0,
+    ROT_ROTATE,
+    ROT_HOLD
+};
 
-    enum
+#define DIR_R 0x1
+#define DIR_L 0x2
+
+static int   rot_dir;
+static float rot_val;
+
+static void rot_init(void)
+{
+    rot_val = 0.0f;
+    rot_dir = 0;
+}
+
+static void rot_set(int dir, float value, int exclusive)
+{
+    rot_val = value;
+
+    if (exclusive)
+        rot_dir  = dir;
+    else
+        rot_dir |= dir;
+}
+
+static void rot_clr(int dir)
+{
+    rot_dir &= ~dir;
+}
+
+static int rot_get(float *v)
+{
+    *v = 0.0f;
+
+    if ((rot_dir & DIR_R) && (rot_dir & DIR_L))
     {
-        DIR_R = 1,
-        DIR_L
-    } d;
-} view_rotate;
+        return ROT_HOLD;
+    }
+    else if (rot_dir & DIR_R)
+    {
+        *v = +rot_val;
+        return ROT_ROTATE;
+    }
+    else if (rot_dir & DIR_L)
+    {
+        *v = -rot_val;
+        return ROT_ROTATE;
+    }
+    return ROT_NONE;
+}
 
-#define VIEWR_SET_R(v) do {                                    \
-    view_rotate.R = (v);                                       \
-    view_rotate.d = (v) ? DIR_R : (view_rotate.L ? DIR_L : 0); \
-} while (0)
-
-#define VIEWR_SET_L(v) do {                                    \
-    view_rotate.L = (v);                                       \
-    view_rotate.d = (v) ? DIR_L : (view_rotate.R ? DIR_R : 0); \
-} while (0)
+/*---------------------------------------------------------------------------*/
 
 static int fast_rotate;
 static int show_hud;
@@ -303,7 +322,7 @@ static int play_loop_gui(void)
 {
     int id;
 
-    if ((id = gui_label(0, _("GO!"), GUI_LRG, GUI_ALL, gui_blu, gui_grn)))
+    if ((id = gui_label(0, _("GO!"), GUI_LRG, gui_blu, gui_grn)))
     {
         gui_layout(id, 0, 0);
         gui_pulse(id, 1.2f);
@@ -314,8 +333,7 @@ static int play_loop_gui(void)
 
 static int play_loop_enter(struct state *st, struct state *prev)
 {
-    VIEWR_SET_R(0);
-    VIEWR_SET_L(0);
+    rot_init();
     fast_rotate = 0;
 
     if (prev == &st_pause)
@@ -348,12 +366,29 @@ static void play_loop_timer(int id, float dt)
                (float) config_get_d(CONFIG_ROTATE_FAST) / 100.0f :
                (float) config_get_d(CONFIG_ROTATE_SLOW) / 100.0f);
 
+    float r = 0.0f;
+
     gui_timer(id, dt);
     hud_timer(dt);
-    game_set_rot(view_rotate.d == DIR_R ?
-                 view_rotate.R * k :
-                 view_rotate.L * k);
-    game_set_cam(config_get_d(CONFIG_CAMERA));
+
+    switch (rot_get(&r))
+    {
+    case ROT_HOLD:
+        /*
+         * Cam 3 could be anything. But let's assume it's a manual cam
+         * and holding down both rotation buttons freezes the camera
+         * rotation.
+         */
+        game_set_rot(0.0f);
+        game_set_cam(CAM_3);
+        break;
+
+    case ROT_ROTATE:
+    case ROT_NONE:
+        game_set_rot(r * k);
+        game_set_cam(config_get_d(CONFIG_CAMERA));
+        break;
+    }
 
     game_step_fade(dt);
 
@@ -384,105 +419,46 @@ static void play_loop_timer(int id, float dt)
     }
 }
 
-#ifdef __MAEMO__
-#define TOUCH_SENSITIVITY       20
-static int button_pressed = 0;
-static int button_pressed_x = 0;
-static int button_pressed_y = 0;
-static int dragging = 0;
-
-static void maemo_play_loop_point(int id, int x, int y, int dx, int dy)
-{
-    if (!dragging && button_pressed)
-    {
-        /* check how much we've moved */
-        if (abs(x - button_pressed_x) > TOUCH_SENSITIVITY ||
-            abs(y - button_pressed_y) > TOUCH_SENSITIVITY)
-            dragging = 1;
-    }
-
-    if (dragging)
-    {
-        if (dx > 0)
-        {
-            view_rotate.d = DIR_R;
-            view_rotate.R = dx / 15.0;
-        }
-        else
-        {
-            view_rotate.d = DIR_L;
-            view_rotate.L = dx / 15.0;
-        }
-    }
-}
-
-static void maemo_play_loop_click(int b, int d)
-{
-    if (d)
-    {
-        int y;
-        SDL_GetMouseState(&button_pressed_x, &y);
-        button_pressed_y = -y + config_get_d(CONFIG_HEIGHT);
-    }
-    else
-    {
-        if (button_pressed && !dragging)
-            goto_state(&st_pause);
-        VIEWR_SET_R(0);
-        VIEWR_SET_L(0);
-    }
-    button_pressed = d;
-    dragging = 0;
-}
-#endif
-
 static void play_loop_point(int id, int x, int y, int dx, int dy)
 {
-#ifndef __MAEMO__
     game_set_pos(dx, dy);
-#else
-    maemo_play_loop_point(id, x, y, dx, dy);
-#endif
 }
 
 static void play_loop_stick(int id, int a, float v, int bump)
 {
-    if (config_tst_d(CONFIG_JOYSTICK_AXIS_X, a))
+    if (config_tst_d(CONFIG_JOYSTICK_AXIS_X0, a))
         game_set_z(v);
-    if (config_tst_d(CONFIG_JOYSTICK_AXIS_Y, a))
+    if (config_tst_d(CONFIG_JOYSTICK_AXIS_Y0, a))
         game_set_x(v);
-    if (config_tst_d(CONFIG_JOYSTICK_AXIS_U, a))
+    if (config_tst_d(CONFIG_JOYSTICK_AXIS_X1, a))
     {
-        VIEWR_SET_R(0);
-        VIEWR_SET_L(0);
-
-        if (v > 0) VIEWR_SET_R(v);
-        if (v < 0) VIEWR_SET_L(v);
+        if      (v > 0.0f)
+            rot_set(DIR_R, +v, 1);
+        else if (v < 0.0f)
+            rot_set(DIR_L, -v, 1);
+        else
+            rot_clr(DIR_R | DIR_L);
     }
 }
 
 static int play_loop_click(int b, int d)
 {
-#ifndef __MAEMO__
     if (d)
     {
         if (config_tst_d(CONFIG_MOUSE_CAMERA_R, b))
-            VIEWR_SET_R(+1);
+            rot_set(DIR_R, 1.0f, 0);
         if (config_tst_d(CONFIG_MOUSE_CAMERA_L, b))
-            VIEWR_SET_L(-1);
+            rot_set(DIR_L, 1.0f, 0);
 
         click_camera(b);
     }
     else
     {
         if (config_tst_d(CONFIG_MOUSE_CAMERA_R, b))
-            VIEWR_SET_R(0);
+            rot_clr(DIR_R);
         if (config_tst_d(CONFIG_MOUSE_CAMERA_L, b))
-            VIEWR_SET_L(0);
+            rot_clr(DIR_L);
     }
-#else
-    maemo_play_loop_click(b, d);
-#endif
 
     return 1;
 }
@@ -492,9 +468,9 @@ static int play_loop_keybd(int c, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_KEY_CAMERA_R, c))
-            VIEWR_SET_R(+1);
+            rot_set(DIR_R, 1.0f, 0);
         if (config_tst_d(CONFIG_KEY_CAMERA_L, c))
-            VIEWR_SET_L(-1);
+            rot_set(DIR_L, 1.0f, 0);
         if (config_tst_d(CONFIG_KEY_ROTATE_FAST, c))
             fast_rotate = 1;
 
@@ -506,23 +482,23 @@ static int play_loop_keybd(int c, int d)
             if (progress_same())
                 goto_state(&st_play_ready);
         }
-        if (config_tst_d(CONFIG_KEY_PAUSE, c))
+        if (c == KEY_EXIT)
             goto_state(&st_pause);
     }
     else
     {
         if (config_tst_d(CONFIG_KEY_CAMERA_R, c))
-            VIEWR_SET_R(0);
+            rot_clr(DIR_R);
         if (config_tst_d(CONFIG_KEY_CAMERA_L, c))
-            VIEWR_SET_L(0);
+            rot_clr(DIR_L);
         if (config_tst_d(CONFIG_KEY_ROTATE_FAST, c))
             fast_rotate = 0;
     }
 
-    if (d && c == SDLK_F12 && config_cheat())
+    if (d && c == KEY_LOOKAROUND && config_cheat())
         return goto_state(&st_look);
 
-    if (d && c == SDLK_F6)
+    if (d && c == KEY_POSE)
         show_hud = !show_hud;
 
     if (d && c == SDLK_c && config_cheat())
@@ -537,25 +513,25 @@ static int play_loop_buttn(int b, int d)
 {
     if (d == 1)
     {
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            pause_or_exit();
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            goto_state(&st_pause);
 
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R, b))
-            VIEWR_SET_R(+1);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L, b))
-            VIEWR_SET_L(-1);
-        if (config_tst_d(CONFIG_JOYSTICK_ROTATE_FAST, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R1, b))
+            rot_set(DIR_R, 1.0f, 0);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L1, b))
+            rot_set(DIR_L, 1.0f, 0);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L2, b))
             fast_rotate = 1;
 
         buttn_camera(b);
     }
     else
     {
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R, b))
-            VIEWR_SET_R(0);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L, b))
-            VIEWR_SET_L(0);
-        if (config_tst_d(CONFIG_JOYSTICK_ROTATE_FAST, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_R1, b))
+            rot_clr(DIR_R);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L1, b))
+            rot_clr(DIR_L);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_L2, b))
             fast_rotate = 0;
     }
     return 1;
@@ -584,8 +560,8 @@ static void look_paint(int id, float t)
 
 static void look_point(int id, int x, int y, int dx, int dy)
 {
-    phi   +=  90.0f * dy / config_get_d(CONFIG_HEIGHT);
-    theta += 180.0f * dx / config_get_d(CONFIG_WIDTH);
+    phi   +=  90.0f * dy / video.device_h;
+    theta += 180.0f * dx / video.device_w;
 
     if (phi > +90.0f) phi = +90.0f;
     if (phi < -90.0f) phi = -90.0f;
@@ -598,15 +574,18 @@ static void look_point(int id, int x, int y, int dx, int dy)
 
 static int look_keybd(int c, int d)
 {
-    if (d && c == SDLK_F12)
-        return goto_state(&st_play_loop);
+    if (d)
+    {
+        if (c == KEY_EXIT || c == KEY_LOOKAROUND)
+            return goto_state(&st_play_loop);
+    }
 
     return 1;
 }
 
 static int look_buttn(int b, int d)
 {
-    if (d && config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+    if (d && (config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b)))
         return goto_state(&st_play_loop);
 
     return 1;
