@@ -3,6 +3,9 @@
 
 #include "state.h"
 
+extern struct state st_save;
+extern struct state st_clobber;
+
 int goto_save(struct state *, struct state *);
 
 #endif
