@@ -22,6 +22,7 @@
 #include "ball.h"
 #include "hole.h"
 #include "hud.h"
+#include "hmd.h"
 #include "image.h"
 #include "audio.h"
 #include "config.h"
@@ -160,7 +161,7 @@ static void game_draw_balls(struct s_rend *rend,
 
     int ui;
 
-    glEnable(GL_COLOR_MATERIAL);
+    r_color_mtrl(rend, 1);
 
     for (ui = curr_party(); ui > 0; ui--)
     {
@@ -211,78 +212,70 @@ static void game_draw_balls(struct s_rend *rend,
     }
 
     glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
-    glDisable(GL_COLOR_MATERIAL);
+    r_color_mtrl(rend, 0);
 }
 
-static void game_draw_goals(struct s_rend *rend, const struct s_base *fp)
+static void game_draw_flags(struct s_rend *rend, const struct s_base *fp)
 {
     int zi;
 
     for (zi = 0; zi < fp->zc; zi++)
-    {
-        glPushMatrix();
-        {
-            glTranslatef(fp->zv[zi].p[0],
-                         fp->zv[zi].p[1],
-                         fp->zv[zi].p[2]);
-            flag_draw(rend);
-        }
-        glPopMatrix();
-    }
+        flag_draw(rend, fp->zv[zi].p);
 }
 
-static void game_draw_jumps(struct s_rend *rend, const struct s_base *fp)
+static void game_draw_beams(struct s_rend *rend, const struct s_base *bp,
+                                                 const struct s_vary *vp)
 {
-    float t = 0.001f * SDL_GetTicks();
-    int ji;
+    static const GLfloat jump_c[2][4]    =  {{ 0.7f, 0.5f, 1.0f, 0.5f },
+                                             { 0.7f, 0.5f, 1.0f, 0.8f }};
+    static const GLfloat swch_c[2][2][4] = {{{ 1.0f, 0.0f, 0.0f, 0.5f },
+                                             { 1.0f, 0.0f, 0.0f, 0.8f }},
+                                            {{ 0.0f, 1.0f, 0.0f, 0.5f },
+                                             { 0.0f, 1.0f, 0.0f, 0.8f }}};
 
-    for (ji = 0; ji < fp->jc; ji++)
+    int i;
+
+    /* Jump beams */
+
+    for (i = 0; i < bp->jc; i++)
+        beam_draw(rend, bp->jv[i].p, jump_c[jump_e ? 0 : 1],
+                        bp->jv[i].r, 2.0f);
+
+    /* Switch beams */
+
+    for (i = 0; i < vp->xc; i++)
     {
-        glPushMatrix();
-        {
-            glTranslatef(fp->jv[ji].p[0],
-                         fp->jv[ji].p[1],
-                         fp->jv[ji].p[2]);
+        struct v_swch *xp = vp->xv + i;
 
-            glScalef(fp->jv[ji].r, 1.f, fp->jv[ji].r);
-            jump_draw(rend, t, !jump_e);
-        }
-        glPopMatrix();
-    }
-}
-
-static void game_draw_swchs(struct s_rend *rend, const struct s_vary *fp)
-{
-    int xi;
-
-    for (xi = 0; xi < fp->xc; xi++)
-    {
-        struct v_swch *xp = fp->xv + xi;
-
-        if (xp->base->i)
-            continue;
-
-        glPushMatrix();
-        {
-            glTranslatef(xp->base->p[0],
-                         xp->base->p[1],
-                         xp->base->p[2]);
-
-            glScalef(xp->base->r, 1.f, xp->base->r);
-            swch_draw(rend, xp->f, xp->e);
-        }
-        glPopMatrix();
+        if (!xp->base->i)
+            beam_draw(rend, xp->base->p, swch_c[xp->f][xp->e],
+                            xp->base->r, 2.0f);
     }
 }
 
 /*---------------------------------------------------------------------------*/
 
+static void game_shadow_conf(int enable)
+{
+    if (enable && config_get_d(CONFIG_SHADOW))
+    {
+        tex_env_select(&tex_env_shadow_clip,
+                       &tex_env_shadow,
+                       &tex_env_default,
+                       NULL);
+    }
+    else
+    {
+        tex_env_active(&tex_env_default);
+    }
+}
+
 void game_draw(int pose, float t)
 {
     const float light_p[4] = { 8.f, 32.f, 8.f, 0.f };
 
     struct s_draw *fp = &file.draw;
-    struct s_rend rend = { NULL };
+    struct s_rend rend;
 
     float fov = FOV;
 
@@ -291,33 +284,38 @@ void game_draw(int pose, float t)
 
     fp->shadow_ui = ball;
 
-    sol_draw_enable(&rend);
+    game_shadow_conf(1);
+    r_draw_enable(&rend);
 
     if (jump_b) fov *= 2.0f * fabsf(jump_dt - 0.5f);
 
     video_push_persp(fov, 0.1f, FAR_DIST);
     glPushMatrix();
     {
-        float T[16], M[16], v[3], rx, ry;
+        float T[16], M[16], v[3], c[3];
 
-        m_view(T, view_c, view_p, view_e[1]);
+        /* In VR, move the view center up to keep the viewer level. */
+
+        v_cpy(c, view_c);
+
+        if (hmd_stat())
+            c[1] += view_dy;
+
+        video_calc_view(T, c, view_p, view_e[1]);
         m_xps(M, T);
 
-        v_sub(v, view_c, view_p);
-
-        rx = V_DEG(fatan2f(-v[1], fsqrtf(v[0] * v[0] + v[2] * v[2])));
-        ry = V_DEG(fatan2f(+v[0], -v[2]));
+        v_sub(v, c, view_p);
 
         glTranslatef(0.f, 0.f, -v_len(v));
         glMultMatrixf(M);
-        glTranslatef(-view_c[0], -view_c[1], -view_c[2]);
+        glTranslatef(-c[0], -c[1], -c[2]);
 
         /* Center the skybox about the position of the camera. */
 
         glPushMatrix();
         {
             glTranslatef(view_p[0], view_p[1], view_p[2]);
-            back_draw(&rend, 0);
+            back_draw(&rend);
         }
         glPopMatrix();
 
@@ -339,22 +337,22 @@ void game_draw(int pose, float t)
             game_draw_vect(&rend, fp->vary);
         }
 
-        glEnable(GL_COLOR_MATERIAL);
         glDisable(GL_LIGHTING);
         glDepthMask(GL_FALSE);
         {
-            game_draw_goals(&rend, fp->base);
-            game_draw_jumps(&rend, fp->base);
-            game_draw_swchs(&rend, fp->vary);
+            game_draw_flags(&rend, fp->base);
+            game_draw_beams(&rend, fp->base, fp->vary);
         }
         glDepthMask(GL_TRUE);
         glEnable(GL_LIGHTING);
-        glDisable(GL_COLOR_MATERIAL);
+
+        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
     }
     glPopMatrix();
     video_pop_matrix();
 
-    sol_draw_disable(&rend);
+    r_draw_disable(&rend);
+    game_shadow_conf(0);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -443,7 +441,7 @@ static int game_update_state(float dt)
 
     /* Test for a switch. */
 
-    if (sol_swch_test(fp, ball) == SWCH_INSIDE)
+    if (sol_swch_test(fp, NULL, ball) == SWCH_INSIDE)
         audio_play(AUD_SWITCH, 1.f);
 
     /* Test for a jump. */
@@ -465,7 +463,7 @@ static int game_update_state(float dt)
 
     /* Test for fall-out. */
 
-    if (fp->uv[ball].p[1] < -10.f)
+    if (file.base.vc == 0 || fp->uv[ball].p[1] < file.base.vv[0].p[1])
         return GAME_FALL;
 
     /* Test for a goal or stop. */
@@ -524,7 +522,7 @@ int game_step(const float g[3], float dt)
 
         /* Handle a jump. */
 
-        if (0.5 < jump_dt)
+        if (0.5f < jump_dt)
         {
             fp->uv[ball].p[0] = jump_p[0];
             fp->uv[ball].p[1] = jump_p[1];
@@ -545,7 +543,7 @@ int game_step(const float g[3], float dt)
 
         for (i = 0; i < n; i++)
         {
-            d = sol_step(fp, g, t, ball, &m);
+            d = sol_step(fp, NULL, g, t, ball, &m);
 
             if (b < d)
                 b = d;
@@ -555,8 +553,8 @@ int game_step(const float g[3], float dt)
 
         /* Mix the sound of a ball bounce. */
 
-        if (b > 0.5)
-            audio_play(AUD_BUMP, (float) (b - 0.5) * 2.0f);
+        if (b > 0.5f)
+            audio_play(AUD_BUMP, (b - 0.5f) * 2.0f);
     }
 
     game_update_view(dt);
@@ -594,8 +592,8 @@ void game_set_mag(int d)
 {
     view_m -= (float) (1.f * d) / config_get_d(CONFIG_MOUSE_SENSE);
 
-    if (view_m < 0.25)
-        view_m = 0.25;
+    if (view_m < 0.25f)
+        view_m = 0.25f;
 }
 
 void game_set_fly(float k)
@@ -616,7 +614,11 @@ void game_set_fly(float k)
 
     v_cpy(view_e[0], x);
     v_cpy(view_e[1], y);
-    v_sub(view_e[2], fp->uv[ball].p, fp->base->zv[0].p);
+
+    if (fp->base->zc > 0)
+        v_sub(view_e[2], fp->uv[ball].p, fp->base->zv[0].p);
+    else
+        v_cpy(view_e[2], z);
 
     if (fabs(v_dot(view_e[1], view_e[2])) > 0.999)
         v_cpy(view_e[2], z);
