@@ -54,10 +54,10 @@ static int score_card(const char  *title,
 {
     int id, jd, kd, ld;
 
-    int p1 = (curr_party() >= 1) ? 1 : 0, l1 = (curr_party() == 1) ? 1 : 0;
-    int p2 = (curr_party() >= 2) ? 1 : 0, l2 = (curr_party() == 2) ? 1 : 0;
-    int p3 = (curr_party() >= 3) ? 1 : 0, l3 = (curr_party() == 3) ? 1 : 0;
-    int p4 = (curr_party() >= 4) ? 1 : 0, l4 = (curr_party() == 4) ? 1 : 0;
+    int p1 = (curr_party() >= 1) ? 1 : 0;
+    int p2 = (curr_party() >= 2) ? 1 : 0;
+    int p3 = (curr_party() >= 3) ? 1 : 0;
+    int p4 = (curr_party() >= 4) ? 1 : 0;
 
     int i;
     int n = curr_count() - 1;
@@ -65,40 +65,53 @@ static int score_card(const char  *title,
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, title, GUI_MED, GUI_ALL, c0, c1);
+        gui_label(id, title, GUI_MED, c0, c1);
         gui_space(id);
 
         if ((jd = gui_hstack(id)))
         {
             if ((kd = gui_varray(jd)))
             {
-                if (p1) gui_label(kd, _("O"),         0, GUI_NE, 0, 0);
-                if (p1) gui_label(kd, hole_out(0), 0, 0,           gui_wht, gui_wht);
-                if (p1) gui_label(kd, hole_out(1), 0, GUI_SE * l1, gui_red, gui_wht);
-                if (p2) gui_label(kd, hole_out(2), 0, GUI_SE * l2, gui_grn, gui_wht);
-                if (p3) gui_label(kd, hole_out(3), 0, GUI_SE * l3, gui_blu, gui_wht);
-                if (p4) gui_label(kd, hole_out(4), 0, GUI_SE * l4, gui_yel, gui_wht);
+                if (p1) gui_label(kd, _("O"),      0, 0, 0);
+                if (p1) gui_label(kd, hole_out(0), 0, gui_wht, gui_wht);
+                if (p1) gui_label(kd, hole_out(1), 0, gui_red, gui_wht);
+                if (p2) gui_label(kd, hole_out(2), 0, gui_grn, gui_wht);
+                if (p3) gui_label(kd, hole_out(3), 0, gui_blu, gui_wht);
+                if (p4) gui_label(kd, hole_out(4), 0, gui_yel, gui_wht);
+
+                gui_set_rect(kd, GUI_RGT);
             }
 
             if ((kd = gui_harray(jd)))
+            {
                 for (i = m; i > 0; i--)
                     if ((ld = gui_varray(kd)))
                     {
-                        if (p1) gui_label(ld, number(i), 0, (i == 1) ? GUI_NW : 0, 0, 0);
-                        if (p1) gui_label(ld, hole_score(i, 0), 0, 0, gui_wht, gui_wht);
-                        if (p1) gui_label(ld, hole_score(i, 1), 0, 0, gui_red, gui_wht);
-                        if (p2) gui_label(ld, hole_score(i, 2), 0, 0, gui_grn, gui_wht);
-                        if (p3) gui_label(ld, hole_score(i, 3), 0, 0, gui_blu, gui_wht);
-                        if (p4) gui_label(ld, hole_score(i, 4), 0, 0, gui_yel, gui_wht);
+                        if (p1) gui_label(ld, number(i), 0, 0, 0);
+                        if (p1) gui_label(ld, hole_score(i, 0), 0, gui_wht, gui_wht);
+                        if (p1) gui_label(ld, hole_score(i, 1), 0, gui_red, gui_wht);
+                        if (p2) gui_label(ld, hole_score(i, 2), 0, gui_grn, gui_wht);
+                        if (p3) gui_label(ld, hole_score(i, 3), 0, gui_blu, gui_wht);
+                        if (p4) gui_label(ld, hole_score(i, 4), 0, gui_yel, gui_wht);
                     }
-            if ((kd = gui_varray(jd)))
+
+                gui_set_rect(kd, GUI_LFT);
+            }
+
+            if ((kd = gui_vstack(jd)))
             {
-                gui_filler(kd);
-                if (p1) gui_label(kd, _("Par"), 0, GUI_NW,      gui_wht, gui_wht);
-                if (p1) gui_label(kd, _("P1"),  0, GUI_SW * l1, gui_red, gui_wht);
-                if (p2) gui_label(kd, _("P2"),  0, GUI_SW * l2, gui_grn, gui_wht);
-                if (p3) gui_label(kd, _("P3"),  0, GUI_SW * l3, gui_blu, gui_wht);
-                if (p4) gui_label(kd, _("P4"),  0, GUI_SW * l4, gui_yel, gui_wht);
+                gui_space(kd);
+
+                if ((ld = gui_varray(kd)))
+                {
+                    if (p1) gui_label(ld, _("Par"), 0, gui_wht, gui_wht);
+                    if (p1) gui_label(ld, _("P1"),  0, gui_red, gui_wht);
+                    if (p2) gui_label(ld, _("P2"),  0, gui_grn, gui_wht);
+                    if (p3) gui_label(ld, _("P3"),  0, gui_blu, gui_wht);
+                    if (p4) gui_label(ld, _("P4"),  0, gui_yel, gui_wht);
+
+                    gui_set_rect(ld, GUI_ALL);
+                }
             }
         }
 
@@ -108,41 +121,58 @@ static int score_card(const char  *title,
         {
             if ((kd = gui_varray(jd)))
             {
-                if (p1) gui_label(kd, _("Tot"),    0, GUI_TOP, 0, 0);
-                if (p1) gui_label(kd, hole_tot(0), 0, 0,           gui_wht, gui_wht);
-                if (p1) gui_label(kd, hole_tot(1), 0, GUI_BOT * l1, gui_red, gui_wht);
-                if (p2) gui_label(kd, hole_tot(2), 0, GUI_BOT * l2, gui_grn, gui_wht);
-                if (p3) gui_label(kd, hole_tot(3), 0, GUI_BOT * l3, gui_blu, gui_wht);
-                if (p4) gui_label(kd, hole_tot(4), 0, GUI_BOT * l4, gui_yel, gui_wht);
+                if (p1) gui_label(kd, _("Tot"),    0, 0, 0);
+                if (p1) gui_label(kd, hole_tot(0), 0, gui_wht, gui_wht);
+                if (p1) gui_label(kd, hole_tot(1), 0, gui_red, gui_wht);
+                if (p2) gui_label(kd, hole_tot(2), 0, gui_grn, gui_wht);
+                if (p3) gui_label(kd, hole_tot(3), 0, gui_blu, gui_wht);
+                if (p4) gui_label(kd, hole_tot(4), 0, gui_yel, gui_wht);
+
+                gui_set_rect(kd, GUI_ALL);
             }
+
             if ((kd = gui_varray(jd)))
             {
-                if (p1) gui_label(kd, _("I"),     0, GUI_NE, 0, 0);
-                if (p1) gui_label(kd, hole_in(0), 0, 0,           gui_wht, gui_wht);
-                if (p1) gui_label(kd, hole_in(1), 0, GUI_SE * l1, gui_red, gui_wht);
-                if (p2) gui_label(kd, hole_in(2), 0, GUI_SE * l2, gui_grn, gui_wht);
-                if (p3) gui_label(kd, hole_in(3), 0, GUI_SE * l3, gui_blu, gui_wht);
-                if (p4) gui_label(kd, hole_in(4), 0, GUI_SE * l4, gui_yel, gui_wht);
+                if (p1) gui_label(kd, _("I"),     0, 0, 0);
+                if (p1) gui_label(kd, hole_in(0), 0, gui_wht, gui_wht);
+                if (p1) gui_label(kd, hole_in(1), 0, gui_red, gui_wht);
+                if (p2) gui_label(kd, hole_in(2), 0, gui_grn, gui_wht);
+                if (p3) gui_label(kd, hole_in(3), 0, gui_blu, gui_wht);
+                if (p4) gui_label(kd, hole_in(4), 0, gui_yel, gui_wht);
+
+                gui_set_rect(kd, GUI_RGT);
             }
+
             if ((kd = gui_harray(jd)))
+            {
                 for (i = n; i > m; i--)
                     if ((ld = gui_varray(kd)))
                     {
-                        if (p1) gui_label(ld, number(i), 0, (i == m+1) ? GUI_NW : 0, 0, 0);
-                        if (p1) gui_label(ld, hole_score(i, 0), 0, 0, gui_wht, gui_wht);
-                        if (p1) gui_label(ld, hole_score(i, 1), 0, 0, gui_red, gui_wht);
-                        if (p2) gui_label(ld, hole_score(i, 2), 0, 0, gui_grn, gui_wht);
-                        if (p3) gui_label(ld, hole_score(i, 3), 0, 0, gui_blu, gui_wht);
-                        if (p4) gui_label(ld, hole_score(i, 4), 0, 0, gui_yel, gui_wht);
+                        if (p1) gui_label(ld, number(i), 0, 0, 0);
+                        if (p1) gui_label(ld, hole_score(i, 0), 0, gui_wht, gui_wht);
+                        if (p1) gui_label(ld, hole_score(i, 1), 0, gui_red, gui_wht);
+                        if (p2) gui_label(ld, hole_score(i, 2), 0, gui_grn, gui_wht);
+                        if (p3) gui_label(ld, hole_score(i, 3), 0, gui_blu, gui_wht);
+                        if (p4) gui_label(ld, hole_score(i, 4), 0, gui_yel, gui_wht);
                     }
-            if ((kd = gui_varray(jd)))
+
+                gui_set_rect(kd, GUI_LFT);
+            }
+
+            if ((kd = gui_vstack(jd)))
             {
-                gui_filler(kd);
-                if (p1) gui_label(kd, _("Par"), 0, GUI_NW,      gui_wht, gui_wht);
-                if (p1) gui_label(kd, _("P1"),  0, GUI_SW * l1, gui_red, gui_wht);
-                if (p2) gui_label(kd, _("P2"),  0, GUI_SW * l2, gui_grn, gui_wht);
-                if (p3) gui_label(kd, _("P3"),  0, GUI_SW * l3, gui_blu, gui_wht);
-                if (p4) gui_label(kd, _("P4"),  0, GUI_SW * l4, gui_yel, gui_wht);
+                gui_space(kd);
+
+                if ((ld = gui_varray(kd)))
+                {
+                    if (p1) gui_label(ld, _("Par"), 0, gui_wht, gui_wht);
+                    if (p1) gui_label(ld, _("P1"),  0, gui_red, gui_wht);
+                    if (p2) gui_label(ld, _("P2"),  0, gui_grn, gui_wht);
+                    if (p3) gui_label(ld, _("P3"),  0, gui_blu, gui_wht);
+                    if (p4) gui_label(ld, _("P4"),  0, gui_yel, gui_wht);
+
+                    gui_set_rect(ld, GUI_ALL);
+                }
             }
         }
 
@@ -196,7 +226,7 @@ static int title_enter(struct state *st, struct state *prev)
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, "Neverputt", GUI_LRG, GUI_ALL, 0, 0);
+        gui_label(id, "Neverputt", GUI_LRG, 0, 0);
         gui_space(id);
 
         if ((jd = gui_harray(id)))
@@ -205,9 +235,9 @@ static int title_enter(struct state *st, struct state *prev)
 
             if ((kd = gui_varray(jd)))
             {
-                gui_start(kd, sgettext("menu^Play"),    GUI_MED, TITLE_PLAY, 1);
-                gui_state(kd, sgettext("menu^Options"), GUI_MED, TITLE_CONF, 0);
-                gui_state(kd, sgettext("menu^Exit"),    GUI_MED, TITLE_EXIT, 0);
+                gui_start(kd, gt_prefix("menu^Play"),    GUI_MED, TITLE_PLAY, 1);
+                gui_state(kd, gt_prefix("menu^Options"), GUI_MED, TITLE_CONF, 0);
+                gui_state(kd, gt_prefix("menu^Exit"),    GUI_MED, TITLE_EXIT, 0);
             }
 
             gui_filler(jd);
@@ -224,6 +254,15 @@ static int title_enter(struct state *st, struct state *prev)
 static void title_leave(struct state *st, struct state *next, int id)
 {
     gui_delete(id);
+
+    if (next == &st_conf)
+    {
+        /*
+         * This is ugly, but better than stupidly deleting stuff using
+         * object names from a previous GL context.
+         */
+        course_free();
+    }
 }
 
 static void title_paint(int id, float t)
@@ -249,7 +288,7 @@ static void title_point(int id, int x, int y, int dx, int dy)
 
 static int title_click(int b, int d)
 {
-    return d && b == SDL_BUTTON_LEFT ? title_action(gui_token(gui_click())) : 1;
+    return gui_click(b, d) ? title_action(gui_token(gui_active())) : 1;
 }
 
 static int title_buttn(int b, int d)
@@ -257,8 +296,8 @@ static int title_buttn(int b, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
-            return title_action(gui_token(gui_click()));
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+            return title_action(gui_token(gui_active()));
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return title_action(TITLE_EXIT);
     }
     return 1;
@@ -273,6 +312,8 @@ static int shot_id;
 
 static int course_action(int i)
 {
+    audio_play(AUD_MENU, 1.0f);
+
     if (course_exists(i))
     {
         course_goto(i);
@@ -303,8 +344,8 @@ static int comp_rows(int n)
 
 static int course_enter(struct state *st, struct state *prev)
 {
-    int w = config_get_d(CONFIG_WIDTH);
-    int h = config_get_d(CONFIG_HEIGHT);
+    int w = video.device_w;
+    int h = video.device_h;
 
     int id, jd, kd, ld, md;
 
@@ -317,7 +358,7 @@ static int course_enter(struct state *st, struct state *prev)
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, _("Select Course"), GUI_MED, GUI_ALL, 0, 0);
+        gui_label(id, _("Select Course"), GUI_MED, 0, 0);
         gui_space(id);
 
         if ((jd = gui_hstack(id)))
@@ -340,7 +381,7 @@ static int course_enter(struct state *st, struct state *prev)
                             {
                                 md = gui_image(ld, course_shot(k),
                                                w / 3 / c, h / 3 / r);
-                                gui_active(md, k, 0);
+                                gui_set_state(md, k, 0);
 
                                 if (k == 0)
                                     gui_focus(md);
@@ -354,7 +395,7 @@ static int course_enter(struct state *st, struct state *prev)
         }
 
         gui_space(id);
-        desc_id = gui_multi(id, _(course_desc(0)), GUI_SML, GUI_ALL, gui_yel, gui_wht);
+        desc_id = gui_multi(id, _(course_desc(0)), GUI_SML, gui_yel, gui_wht);
         gui_space(id);
 
         if ((jd = gui_hstack(id)))
@@ -423,7 +464,7 @@ static void course_stick(int id, int a, float v, int bump)
 
 static int course_click(int b, int d)
 {
-    return d && b == SDL_BUTTON_LEFT ? course_action(gui_token(gui_click())) : 1;
+    return gui_click(b, d) ? course_action(gui_token(gui_active())) : 1;
 }
 
 static int course_buttn(int b, int d)
@@ -431,8 +472,8 @@ static int course_buttn(int b, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
-            return course_action(gui_token(gui_click()));
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+            return course_action(gui_token(gui_active()));
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return course_action(COURSE_BACK);
     }
     return 1;
@@ -485,7 +526,7 @@ static int party_enter(struct state *st, struct state *prev)
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, _("Players?"), GUI_MED, GUI_ALL, 0, 0);
+        gui_label(id, _("Players?"), GUI_MED, 0, 0);
         gui_space(id);
 
         if ((jd = gui_harray(id)))
@@ -540,7 +581,7 @@ static void party_point(int id, int x, int y, int dx, int dy)
 
 static int party_click(int b, int d)
 {
-    return d && b == SDL_BUTTON_LEFT ? party_action(gui_token(gui_click())) : 1;
+    return gui_click(b, d) ? party_action(gui_token(gui_active())) : 1;
 }
 
 static int party_buttn(int b, int d)
@@ -548,8 +589,8 @@ static int party_buttn(int b, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
-            return party_action(gui_token(gui_click()));
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+            return party_action(gui_token(gui_active()));
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return party_action(PARTY_B);
     }
     return 1;
@@ -565,14 +606,11 @@ static struct state *st_quit;
 #define PAUSE_CONTINUE 1
 #define PAUSE_QUIT     2
 
-int goto_pause(struct state *s, int e)
+int goto_pause(struct state *s)
 {
     if (curr_state() == &st_pause)
         return 1;
 
-    if (e && !config_tst_d(CONFIG_KEY_PAUSE, SDLK_ESCAPE))
-        return goto_state(s);
-
     st_continue = curr_state();
     st_quit = s;
     paused = 1;
@@ -603,7 +641,7 @@ static int pause_enter(struct state *st, struct state *prev)
 
     if ((id = gui_vstack(0)))
     {
-        td = gui_label(id, _("Paused"), GUI_LRG, GUI_ALL, 0, 0);
+        td = gui_label(id, _("Paused"), GUI_LRG, 0, 0);
         gui_space(id);
 
         if ((jd = gui_harray(id)))
@@ -646,12 +684,12 @@ static void pause_point(int id, int x, int y, int dx, int dy)
 
 static int pause_click(int b, int d)
 {
-    return d && b == SDL_BUTTON_LEFT ? pause_action(gui_token(gui_click())) : 1;
+    return gui_click(b, d) ? pause_action(gui_token(gui_active())) : 1;
 }
 
 static int pause_keybd(int c, int d)
 {
-    if (d && config_tst_d(CONFIG_KEY_PAUSE, c))
+    if (d && c == KEY_EXIT)
         return pause_action(PAUSE_CONTINUE);
     return 1;
 }
@@ -661,8 +699,9 @@ static int pause_buttn(int b, int d)
     if (d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
-            return pause_action(gui_token(gui_click()));
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+            return pause_action(gui_token(gui_active()));
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b) ||
+            config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
             return pause_action(PAUSE_CONTINUE);
     }
     return 1;
@@ -674,8 +713,8 @@ static int shared_keybd(int c, int d)
 {
     if (d)
     {
-        if (config_tst_d(CONFIG_KEY_PAUSE, c))
-            return goto_pause(&st_over, 0);
+        if (c == KEY_EXIT)
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -686,36 +725,41 @@ static int num = 0;
 
 static int next_enter(struct state *st, struct state *prev)
 {
-    int id;
+    int id, jd;
     char str[MAXSTR];
 
     sprintf(str, _("Hole %02d"), curr_hole());
 
     if ((id = gui_vstack(0)))
     {
-        gui_label(id, str, GUI_MED, GUI_ALL, 0, 0);
+        gui_label(id, str, GUI_MED, 0, 0);
         gui_space(id);
 
-        gui_label(id, _("Player"), GUI_SML, GUI_TOP, 0, 0);
-
-        switch (curr_player())
+        if ((jd = gui_vstack(id)))
         {
-        case 1:
-            gui_label(id, "1", GUI_LRG, GUI_BOT, gui_red, gui_wht);
-            if (curr_party() > 1) audio_play(AUD_PLAYER1, 1.f);
-            break;
-        case 2:
-            gui_label(id, "2", GUI_LRG, GUI_BOT, gui_grn, gui_wht);
-            if (curr_party() > 1) audio_play(AUD_PLAYER2, 1.f);
-            break;
-        case 3:
-            gui_label(id, "3", GUI_LRG, GUI_BOT, gui_blu, gui_wht);
-            if (curr_party() > 1) audio_play(AUD_PLAYER3, 1.f);
-            break;
-        case 4:
-            gui_label(id, "4", GUI_LRG, GUI_BOT, gui_yel, gui_wht);
-            if (curr_party() > 1) audio_play(AUD_PLAYER4, 1.f);
-            break;
+            gui_label(jd, _("Player"), GUI_SML, 0, 0);
+
+            switch (curr_player())
+            {
+            case 1:
+                gui_label(jd, "1", GUI_LRG, gui_red, gui_wht);
+                if (curr_party() > 1) audio_play(AUD_PLAYER1, 1.f);
+                break;
+            case 2:
+                gui_label(jd, "2", GUI_LRG, gui_grn, gui_wht);
+                if (curr_party() > 1) audio_play(AUD_PLAYER2, 1.f);
+                break;
+            case 3:
+                gui_label(jd, "3", GUI_LRG, gui_blu, gui_wht);
+                if (curr_party() > 1) audio_play(AUD_PLAYER3, 1.f);
+                break;
+            case 4:
+                gui_label(jd, "4", GUI_LRG, gui_yel, gui_wht);
+                if (curr_party() > 1) audio_play(AUD_PLAYER4, 1.f);
+                break;
+            }
+
+            gui_set_rect(jd, GUI_ALL);
         }
         gui_layout(id, 0, 0);
     }
@@ -761,10 +805,10 @@ static int next_keybd(int c, int d)
 {
     if (d)
     {
-        if (c == SDLK_F12)
+        if (c == KEY_POSE)
             return goto_state(&st_poser);
-        if (config_tst_d(CONFIG_KEY_PAUSE, c))
-            return goto_pause(&st_over, 0);
+        if (c == KEY_EXIT)
+            return goto_pause(&st_over);
         if ('0' <= c && c <= '9')
             num = num * 10 + c - '0';
     }
@@ -777,15 +821,23 @@ static int next_buttn(int b, int d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
         {
-            if (num > 0 && hole_goto(num, -1))
+            if (num > 0)
             {
-                num = 0;
-                return goto_state(&st_next);
+                if (hole_goto(num, -1))
+                {
+                    num = 0;
+                    return goto_state(&st_next);
+                }
+                else
+                {
+                    num = 0;
+                    return 1;
+                }
             }
             return goto_state(&st_flyby);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -809,7 +861,7 @@ static int poser_buttn(int b, int d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_next);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return goto_state(&st_next);
     }
     return 1;
@@ -819,6 +871,8 @@ static int poser_buttn(int b, int d)
 
 static int flyby_enter(struct state *st, struct state *prev)
 {
+    video_hide_cursor();
+
     if (paused)
         paused = 0;
     else
@@ -829,6 +883,7 @@ static int flyby_enter(struct state *st, struct state *prev)
 
 static void flyby_leave(struct state *st, struct state *next, int id)
 {
+    video_show_cursor();
     hud_free();
 }
 
@@ -869,28 +924,15 @@ static int flyby_buttn(int b, int d)
             game_set_fly(0.f);
             return goto_state(&st_stroke);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b) ||
+            config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
 
 /*---------------------------------------------------------------------------*/
 
-#define TOUCH_SENSITIVITY       20
-#define LONG_PRESS_TIME 1000 /* one second */
-#define DOUBLE_TAP_TIME 500
-static enum {
-    BTN_CLEAR,
-    BTN_PRESS1,
-    BTN_CLICKED,
-    BTN_PRESS2,
-    BTN_DRAGGING
-} button_state = BTN_CLEAR;
-static int button_pressed_x = 0;
-static int button_pressed_y = 0;
-static Uint32 button_time = 0;
-
 static int stroke_rotate = 0;
 static int stroke_mag    = 0;
 
@@ -899,7 +941,7 @@ static int stroke_enter(struct state *st, struct state *prev)
     hud_init();
     game_clr_mag();
     config_set_d(CONFIG_CAMERA, 2);
-    video_set_grab(!paused);
+    video_set_grab(1);
 
     if (paused)
         paused = 0;
@@ -912,6 +954,7 @@ static void stroke_leave(struct state *st, struct state *next, int id)
     hud_free();
     video_clr_grab();
     config_set_d(CONFIG_CAMERA, 0);
+    stroke_rotate = 0.0f;
 }
 
 static void stroke_paint(int id, float t)
@@ -926,21 +969,9 @@ static void stroke_timer(int id, float dt)
 
     float k;
 
-    if (button_state == BTN_CLICKED)
-    {
-        int elapsed = SDL_GetTicks() - button_time;
-        if (elapsed > DOUBLE_TAP_TIME)
-        {
-            /* it was a single click: start the simulation */
-            goto_state(&st_roll);
-            button_state = BTN_CLEAR;
-            return;
-        }
-    }
-
     if (SDL_GetModState() & KMOD_SHIFT ||
         (joystick && SDL_JoystickGetButton(joystick,
-                                           config_get_d(CONFIG_JOYSTICK_BUTTON_B))))
+                                           config_get_d(CONFIG_JOYSTICK_BUTTON_X))))
         k = 0.25;
     else
         k = 1.0;
@@ -954,63 +985,21 @@ static void stroke_timer(int id, float dt)
 
 static void stroke_point(int id, int x, int y, int dx, int dy)
 {
-    if (button_state == BTN_PRESS1 ||
-        button_state == BTN_PRESS2)
-    {
-        int duration = SDL_GetTicks() - button_time;
-        /* check how much we've moved */
-        if (abs(x - button_pressed_x) > TOUCH_SENSITIVITY ||
-            abs(y - button_pressed_y) > TOUCH_SENSITIVITY ||
-            duration > LONG_PRESS_TIME)
-            button_state = BTN_DRAGGING;
-    }
-    if (button_state != BTN_DRAGGING) return;
-
     game_set_rot(dx);
     game_set_mag(dy);
 }
 
 static void stroke_stick(int id, int a, float v, int bump)
 {
-    if (config_tst_d(CONFIG_JOYSTICK_AXIS_X, a))
+    if      (config_tst_d(CONFIG_JOYSTICK_AXIS_X0, a))
         stroke_rotate = 6 * v;
-    else if (config_tst_d(CONFIG_JOYSTICK_AXIS_Y, a))
+    else if (config_tst_d(CONFIG_JOYSTICK_AXIS_Y0, a))
         stroke_mag = -6 * v;
 }
 
 static int stroke_click(int b, int d)
 {
-    int ret = 1;
-    Uint32 current_time;
-
-    current_time = SDL_GetTicks();
-    if (d)
-    {
-        int y;
-        SDL_GetMouseState(&button_pressed_x, &y);
-        button_pressed_y = -y + config_get_d(CONFIG_HEIGHT);
-        button_time = current_time;
-        button_state = (button_state == BTN_CLICKED) ? BTN_PRESS2 : BTN_PRESS1;
-    }
-    else
-    {
-        /* is it a double click? */
-        int duration = current_time - button_time;
-        if (button_state == BTN_PRESS2)
-        {
-            button_state = BTN_CLEAR;
-            if (duration < DOUBLE_TAP_TIME)
-                ret = goto_pause(&st_over, 1);
-        }
-        else
-        {
-            if (button_state == BTN_PRESS1 && duration < LONG_PRESS_TIME)
-                button_state = BTN_CLICKED;
-            else
-                button_state = BTN_CLEAR;
-        }
-    }
-    return ret;
+    return (d && b == SDL_BUTTON_LEFT) ? goto_state(&st_roll) : 1;
 }
 
 static int stroke_buttn(int b, int d)
@@ -1019,8 +1008,8 @@ static int stroke_buttn(int b, int d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_roll);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1029,6 +1018,7 @@ static int stroke_buttn(int b, int d)
 
 static int roll_enter(struct state *st, struct state *prev)
 {
+    video_hide_cursor();
     hud_init();
 
     if (paused)
@@ -1041,6 +1031,7 @@ static int roll_enter(struct state *st, struct state *prev)
 
 static void roll_leave(struct state *st, struct state *next, int id)
 {
+    video_show_cursor();
     hud_free();
 }
 
@@ -1066,8 +1057,9 @@ static int roll_buttn(int b, int d)
 {
     if (d)
     {
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b) ||
+            config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1078,7 +1070,7 @@ static int goal_enter(struct state *st, struct state *prev)
 {
     int id;
 
-    if ((id = gui_label(0, _("It's In!"), GUI_MED, GUI_ALL, gui_grn, gui_grn)))
+    if ((id = gui_label(0, _("It's In!"), GUI_MED, gui_grn, gui_grn)))
         gui_layout(id, 0, 0);
 
     if (paused)
@@ -1138,8 +1130,8 @@ static int goal_buttn(int b, int d)
             else
                 goto_state(&st_score);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1208,8 +1200,9 @@ static int stop_buttn(int b, int d)
             else
                 goto_state(&st_score);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b) ||
+            config_tst_d(CONFIG_JOYSTICK_BUTTON_START, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1220,7 +1213,7 @@ static int fall_enter(struct state *st, struct state *prev)
 {
     int id;
 
-    if ((id = gui_label(0, _("1 Stroke Penalty"), GUI_MED, GUI_ALL, gui_blk, gui_red)))
+    if ((id = gui_label(0, _("1 Stroke Penalty"), GUI_MED, gui_blk, gui_red)))
         gui_layout(id, 0, 0);
 
     if (paused)
@@ -1283,8 +1276,8 @@ static int fall_buttn(int b, int d)
             else
                 goto_state(&st_score);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1340,8 +1333,8 @@ static int score_buttn(int b, int d)
             else
                 goto_state(&st_score);
         }
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
-            return goto_pause(&st_over, 1);
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
+            return goto_pause(&st_over);
     }
     return 1;
 }
@@ -1381,7 +1374,7 @@ static int over_buttn(int b, int d)
     {
         if (config_tst_d(CONFIG_JOYSTICK_BUTTON_A, b))
             return goto_state(&st_title);
-        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_EXIT, b))
+        if (config_tst_d(CONFIG_JOYSTICK_BUTTON_B, b))
             return goto_state(&st_title);
     }
     return 1;
