@@ -62,10 +62,10 @@ static ov_callbacks callbacks = {
 /*---------------------------------------------------------------------------*/
 
 #define MIX(d, s) {                           \
-        int n = (int) (d) + (int) (s);        \
-        if      (n >  32767) (d) =  32767;    \
-        else if (n < -32768) (d) = -32768;    \
-        else                 (d) = (short) n; \
+        int T = (int) (d) + (int) (s);        \
+        if      (T >  32767) (d) =  32767;    \
+        else if (T < -32768) (d) = -32768;    \
+        else                 (d) = (short) T; \
     }
 
 static int voice_step(struct voice *V, float volume, Uint8 *stream, int length)
@@ -106,8 +106,8 @@ static int voice_step(struct voice *V, float volume, Uint8 *stream, int length)
 
                     V->amp += V->damp;
 
-                    if (V->amp < 0.0) V->amp = 0.0;
-                    if (V->amp > 1.0) V->amp = 1.0;
+                    if (V->amp < 0.0f) V->amp = 0.0;
+                    if (V->amp > 1.0f) V->amp = 1.0;
                 }
 
             /* Mix stereo audio. */
@@ -123,8 +123,8 @@ static int voice_step(struct voice *V, float volume, Uint8 *stream, int length)
 
                     V->amp += V->damp;
 
-                    if (V->amp < 0.0) V->amp = 0.0;
-                    if (V->amp > 1.0) V->amp = 1.0;
+                    if (V->amp < 0.0f) V->amp = 0.0;
+                    if (V->amp > 1.0f) V->amp = 1.0;
                 }
 
             r -= n;
@@ -173,8 +173,8 @@ static struct voice *voice_init(const char *filename, float a)
                 V->play = 1;
                 V->loop = 0;
 
-                if (V->amp > 1.0) V->amp = 1.0;
-                if (V->amp < 0.0) V->amp = 0.0;
+                if (V->amp > 1.0f) V->amp = 1.0;
+                if (V->amp < 0.0f) V->amp = 0.0;
 
                 /* The file will be closed when the Ogg is cleared. */
             }
@@ -273,7 +273,7 @@ void audio_init(void)
             audio_state = 1;
             SDL_PauseAudio(0);
         }
-        else fprintf(stderr, "%s\n", SDL_GetError());
+        else log_printf("Failure to open audio device (%s)\n", SDL_GetError());
     }
 
     /* Set the initial volumes. */
@@ -312,8 +312,8 @@ void audio_play(const char *filename, float a)
 
                     V->amp = a;
 
-                    if (V->amp > 1.0) V->amp = 1.0;
-                    if (V->amp < 0.0) V->amp = 0.0;
+                    if (V->amp > 1.0f) V->amp = 1.0;
+                    if (V->amp < 0.0f) V->amp = 0.0;
 
                     SDL_UnlockAudio();
                     return;
