@@ -21,6 +21,7 @@
 #include "common.h"
 
 #include "solid_draw.h"
+#include "solid_sim.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -42,15 +43,11 @@ static int solid_flags;
 static int inner_flags;
 static int outer_flags;
 
-static float solid_alpha;
-static float inner_alpha;
-static float outer_alpha;
-
 /*---------------------------------------------------------------------------*/
 
 #define SET(B, v, b) ((v) ? ((B) | (b)) : ((B) & ~(b)))
 
-static int ball_opts(const struct s_base *base, float *alpha)
+static int ball_opts(const struct s_base *base)
 {
     int flags = F_DEPTHTEST;
     int di;
@@ -70,8 +67,6 @@ static int ball_opts(const struct s_base *base, float *alpha)
             flags = SET(flags, atoi(v), F_DEPTHMASK);
         if (strcmp(k, "depthtest") == 0)
             flags = SET(flags, atoi(v), F_DEPTHTEST);
-        if (strcmp(k, "alphatest") == 0)
-            sscanf(v, "%f", alpha);
     }
 
     return flags;
@@ -90,18 +85,14 @@ void ball_init(void)
     inner_flags = 0;
     outer_flags = 0;
 
-    solid_alpha = 1.0f;
-    inner_alpha = 1.0f;
-    outer_alpha = 1.0f;
-
     if ((has_solid = sol_load_full(&solid, solid_file, 0)))
-        solid_flags = ball_opts(&solid.base, &solid_alpha);
+        solid_flags = ball_opts(&solid.base);
 
     if ((has_inner = sol_load_full(&inner, inner_file, 0)))
-        inner_flags = ball_opts(&inner.base, &inner_alpha);
+        inner_flags = ball_opts(&inner.base);
 
     if ((has_outer = sol_load_full(&outer, outer_file, 0)))
-        outer_flags = ball_opts(&outer.base, &outer_alpha);
+        outer_flags = ball_opts(&outer.base);
 
     free(solid_file);
     free(inner_file);
@@ -117,6 +108,13 @@ void ball_free(void)
     has_solid = has_inner = has_outer = 0;
 }
 
+void ball_step(float dt)
+{
+    if (has_solid) sol_move(&solid.vary, NULL, dt);
+    if (has_inner) sol_move(&inner.vary, NULL, dt);
+    if (has_outer) sol_move(&outer.vary, NULL, dt);
+}
+
 /*---------------------------------------------------------------------------*/
 
 static void ball_draw_solid(struct s_rend *rend,
@@ -128,12 +126,6 @@ static void ball_draw_solid(struct s_rend *rend,
         const int mask = (solid_flags & F_DEPTHMASK);
         const int test = (solid_flags & F_DEPTHTEST);
 
-        if (solid_alpha < 1.0f)
-        {
-            glEnable(GL_ALPHA_TEST);
-            glAlphaFunc(GL_GEQUAL, solid_alpha);
-        }
-
         glPushMatrix();
         {
             /* Apply the ball rotation. */
@@ -160,9 +152,6 @@ static void ball_draw_solid(struct s_rend *rend,
             sol_draw(&solid.draw, rend, mask, test);
         }
         glPopMatrix();
-
-        if (solid_alpha < 1.0f)
-            glDisable(GL_ALPHA_TEST);
     }
 }
 
@@ -177,12 +166,6 @@ static void ball_draw_inner(struct s_rend *rend,
         const int mask = (inner_flags & F_DEPTHMASK);
         const int test = (inner_flags & F_DEPTHTEST);
 
-        if (inner_alpha < 1.0f)
-        {
-            glEnable(GL_ALPHA_TEST);
-            glAlphaFunc(GL_GEQUAL, inner_alpha);
-        }
-
         /* Apply the pendulum rotation. */
 
         if (pend)
@@ -216,9 +199,6 @@ static void ball_draw_inner(struct s_rend *rend,
 
         if (pend)
             glPopMatrix();
-
-        if (inner_alpha < 1.0f)
-            glDisable(GL_ALPHA_TEST);
     }
 }
 
@@ -233,12 +213,6 @@ static void ball_draw_outer(struct s_rend *rend,
         const int mask = (outer_flags & F_DEPTHMASK);
         const int test = (outer_flags & F_DEPTHTEST);
 
-        if (outer_alpha < 1.0f)
-        {
-            glEnable(GL_ALPHA_TEST);
-            glAlphaFunc(GL_GEQUAL, outer_alpha);
-        }
-
         /* Apply the pendulum rotation. */
 
         if (pend)
@@ -271,9 +245,6 @@ static void ball_draw_outer(struct s_rend *rend,
 
         if (pend)
             glPopMatrix();
-
-        if (outer_alpha < 1.0f)
-            glDisable(GL_ALPHA_TEST);
     }
 }
 
