@@ -1,8 +1,6 @@
 #ifndef BINARY_H
 #define BINARY_H
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "fs.h"
@@ -20,13 +18,13 @@ void put_index(fs_file, int);
 void put_short(fs_file, short);
 void put_array(fs_file, const float *, size_t);
 
-void get_float(fs_file, float *);
-void get_index(fs_file, int   *);
-void get_short(fs_file, short *);
-void get_array(fs_file, float *, size_t);
+float get_float(fs_file);
+int   get_index(fs_file);
+short get_short(fs_file);
+void  get_array(fs_file, float *, size_t);
 
 void put_string(fs_file fout, const char *);
-void get_string(fs_file fin, char *, int );
+void get_string(fs_file fin, char *, size_t);
 
 /*---------------------------------------------------------------------------*/
 
