@@ -249,6 +249,16 @@ const char *path_next_sep(const char *path)
     return *(path + skip) ? path + skip : NULL;
 }
 
+char *path_normalize(char *path)
+{
+    char *sep = path;
+
+    while ((sep = (char *) path_next_sep(sep)))
+        *sep++ = '/';
+
+    return path;
+}
+
 const char *base_name_sans(const char *name, const char *suffix)
 {
     static char base[MAXSTR];
@@ -309,3 +319,45 @@ int rand_between(int low, int high)
 }
 
 /*---------------------------------------------------------------------------*/
+
+#ifdef _WIN32
+
+/* MinGW hides this from ANSI C. MinGW-w64 doesn't. */
+_CRTIMP int _putenv(const char *envstring);
+
+int set_env_var(const char *name, const char *value)
+{
+    if (name)
+    {
+        char str[MAXSTR];
+
+        if (value)
+            sprintf(str, "%s=%s", name, value);
+        else
+            sprintf(str, "%s=", name);
+
+        return (_putenv(str) == 0);
+    }
+    return 0;
+}
+
+#else
+
+extern int setenv(const char *name, const char *value, int overwrite);
+extern int unsetenv(const char *name);
+
+int set_env_var(const char *name, const char *value)
+{
+    if (name)
+    {
+        if (value)
+            return (setenv(name, value, 1) == 0);
+        else
+            return (unsetenv(name) == 0);
+    }
+    return 0;
+}
+
+#endif
+
+/*---------------------------------------------------------------------------*/
