@@ -22,10 +22,17 @@
 
 #include <time.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdarg.h>
 #include "fs.h"
 
 /* Random stuff. */
 
+#ifndef MAXSTR
+#define MAXSTR 256
+#endif
+
 #ifdef __GNUC__
 #define NULL_TERMINATED __attribute__ ((__sentinel__))
 #else
@@ -34,8 +41,10 @@
 
 /* Math. */
 
-#define MIN(x, y) ((x) < (y) ? (x) : (y))
-#define MAX(x, y) ((x) > (y) ? (x) : (y))
+#define MIN(a, b) ((a) < (b) ? (a) : (b))
+#define MAX(a, b) ((a) > (b) ? (a) : (b))
+
+#define CLAMP(a, b, c) MIN(MAX(a, b), c)
 
 #define SIGN(n) ((n) < 0 ? -1 : ((n) ? +1 : 0))
 #define ROUND(f) ((int) ((f) + 0.5f * SIGN(f)))
@@ -47,7 +56,10 @@ int rand_between(int low, int high);
 
 /* Arrays and strings. */
 
+#ifndef ARRAYSIZE
 #define ARRAYSIZE(a) (sizeof (a) / sizeof ((a)[0]))
+#endif
+
 #define MAXSTRLEN(a) (sizeof (a) - 1)
 
 #define SAFECPY(dst, src) \
@@ -67,7 +79,16 @@ char *concat_string(const char *first, ...) NULL_TERMINATED;
 #define strdup dupe_string
 
 #define str_starts_with(s, h) (strncmp((s), (h), strlen(h)) == 0)
-#define str_ends_with(s, t) (strcmp((s) + strlen(s) - strlen(t), (t)) == 0)
+#define str_ends_with(s, t) ((strlen(s) >= strlen(t)) && strcmp((s) + strlen(s) - strlen(t), (t)) == 0)
+
+/*
+ * Declaring vsnprintf with the C99 signature, even though we're
+ * claiming to be ANSI C. This is probably bad but is not known to not
+ * work.
+ */
+#ifndef __APPLE__
+extern int vsnprintf(char *, size_t, const char *, va_list);
+#endif
 
 /* Time. */
 
@@ -86,6 +107,7 @@ int path_is_sep(int);
 int path_is_abs(const char *);
 
 char *path_join(const char *, const char *);
+char *path_normalize(char *);
 
 const char *path_last_sep(const char *);
 const char *path_next_sep(const char *);
@@ -94,4 +116,8 @@ const char *base_name(const char *name);
 const char *base_name_sans(const char *name, const char *suffix);
 const char *dir_name(const char *name);
 
+/* Environment */
+
+int set_env_var(const char *, const char *);
+
 #endif
