@@ -13,6 +13,7 @@
  */
 
 #include <SDL.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -27,6 +28,7 @@
 /* Integer options. */
 
 int CONFIG_FULLSCREEN;
+int CONFIG_DISPLAY;
 int CONFIG_WIDTH;
 int CONFIG_HEIGHT;
 int CONFIG_STEREO;
@@ -40,8 +42,11 @@ int CONFIG_BACKGROUND;
 int CONFIG_SHADOW;
 int CONFIG_AUDIO_BUFF;
 int CONFIG_MOUSE_SENSE;
+int CONFIG_MOUSE_RESPONSE;
 int CONFIG_MOUSE_INVERT;
 int CONFIG_VSYNC;
+int CONFIG_HMD;
+int CONFIG_HIGHDPI;
 int CONFIG_MOUSE_CAMERA_1;
 int CONFIG_MOUSE_CAMERA_2;
 int CONFIG_MOUSE_CAMERA_3;
@@ -54,26 +59,30 @@ int CONFIG_SOUND_VOLUME;
 int CONFIG_MUSIC_VOLUME;
 int CONFIG_JOYSTICK;
 int CONFIG_JOYSTICK_DEVICE;
-int CONFIG_JOYSTICK_AXIS_X;
-int CONFIG_JOYSTICK_AXIS_Y;
-int CONFIG_JOYSTICK_AXIS_U;
-int CONFIG_JOYSTICK_AXIS_X_INVERT;
-int CONFIG_JOYSTICK_AXIS_Y_INVERT;
-int CONFIG_JOYSTICK_AXIS_U_INVERT;
+int CONFIG_JOYSTICK_RESPONSE;
+int CONFIG_JOYSTICK_AXIS_X0;
+int CONFIG_JOYSTICK_AXIS_Y0;
+int CONFIG_JOYSTICK_AXIS_X1;
+int CONFIG_JOYSTICK_AXIS_Y1;
+int CONFIG_JOYSTICK_AXIS_X0_INVERT;
+int CONFIG_JOYSTICK_AXIS_Y0_INVERT;
+int CONFIG_JOYSTICK_AXIS_X1_INVERT;
+int CONFIG_JOYSTICK_AXIS_Y1_INVERT;
+
 int CONFIG_JOYSTICK_BUTTON_A;
 int CONFIG_JOYSTICK_BUTTON_B;
-int CONFIG_JOYSTICK_BUTTON_R;
-int CONFIG_JOYSTICK_BUTTON_L;
-int CONFIG_JOYSTICK_BUTTON_EXIT;
-int CONFIG_JOYSTICK_CAMERA_1;
-int CONFIG_JOYSTICK_CAMERA_2;
-int CONFIG_JOYSTICK_CAMERA_3;
+int CONFIG_JOYSTICK_BUTTON_X;
+int CONFIG_JOYSTICK_BUTTON_Y;
+int CONFIG_JOYSTICK_BUTTON_L1;
+int CONFIG_JOYSTICK_BUTTON_R1;
+int CONFIG_JOYSTICK_BUTTON_L2;
+int CONFIG_JOYSTICK_BUTTON_R2;
+int CONFIG_JOYSTICK_BUTTON_START;
 int CONFIG_JOYSTICK_DPAD_L;
 int CONFIG_JOYSTICK_DPAD_R;
 int CONFIG_JOYSTICK_DPAD_U;
 int CONFIG_JOYSTICK_DPAD_D;
-int CONFIG_JOYSTICK_CAMERA_TOGGLE;
-int CONFIG_JOYSTICK_ROTATE_FAST;
+
 int CONFIG_KEY_CAMERA_1;
 int CONFIG_KEY_CAMERA_2;
 int CONFIG_KEY_CAMERA_3;
@@ -84,7 +93,6 @@ int CONFIG_KEY_FORWARD;
 int CONFIG_KEY_BACKWARD;
 int CONFIG_KEY_LEFT;
 int CONFIG_KEY_RIGHT;
-int CONFIG_KEY_PAUSE;
 int CONFIG_KEY_RESTART;
 int CONFIG_KEY_SCORE_NEXT;
 int CONFIG_KEY_ROTATE_FAST;
@@ -98,6 +106,10 @@ int CONFIG_CHEAT;
 int CONFIG_STATS;
 int CONFIG_SCREENSHOT;
 int CONFIG_LOCK_GOALS;
+int CONFIG_CAMERA_1_SPEED;
+int CONFIG_CAMERA_2_SPEED;
+int CONFIG_CAMERA_3_SPEED;
+
 
 /* String options. */
 
@@ -105,6 +117,8 @@ int CONFIG_PLAYER;
 int CONFIG_BALL_FILE;
 int CONFIG_WIIMOTE_ADDR;
 int CONFIG_REPLAY_NAME;
+int CONFIG_LANGUAGE;
+int CONFIG_THEME;
 
 /*---------------------------------------------------------------------------*/
 
@@ -115,22 +129,26 @@ static struct
     const int   def;
     int         cur;
 } option_d[] = {
-    { &CONFIG_FULLSCREEN,   "fullscreen",   1 },
+    { &CONFIG_FULLSCREEN,   "fullscreen",   0 },
+    { &CONFIG_DISPLAY,      "display",      0 },
     { &CONFIG_WIDTH,        "width",        800 },
-    { &CONFIG_HEIGHT,       "height",       480 },
+    { &CONFIG_HEIGHT,       "height",       600 },
     { &CONFIG_STEREO,       "stereo",       0 },
     { &CONFIG_CAMERA,       "camera",       0 },
     { &CONFIG_TEXTURES,     "textures",     1 },
     { &CONFIG_REFLECTION,   "reflection",   1 },
     { &CONFIG_MULTISAMPLE,  "multisample",  0 },
-    { &CONFIG_MIPMAP,       "mipmap",       0 },
+    { &CONFIG_MIPMAP,       "mipmap",       1 },
     { &CONFIG_ANISO,        "aniso",        0 },
     { &CONFIG_BACKGROUND,   "background",   1 },
     { &CONFIG_SHADOW,       "shadow",       1 },
     { &CONFIG_AUDIO_BUFF,   "audio_buff",   AUDIO_BUFF_HI },
     { &CONFIG_MOUSE_SENSE,  "mouse_sense",  300 },
+    { &CONFIG_MOUSE_RESPONSE, "mouse_response", 50 },
     { &CONFIG_MOUSE_INVERT, "mouse_invert", 0 },
     { &CONFIG_VSYNC,        "vsync",        1 },
+    { &CONFIG_HMD,          "hmd",          0 },
+    { &CONFIG_HIGHDPI,      "highdpi",      1 },
 
     { &CONFIG_MOUSE_CAMERA_1,      "mouse_camera_1",      0 },
     { &CONFIG_MOUSE_CAMERA_2,      "mouse_camera_2",      0 },
@@ -144,32 +162,35 @@ static struct
     { &CONFIG_SOUND_VOLUME, "sound_volume", 10 },
     { &CONFIG_MUSIC_VOLUME, "music_volume", 6 },
 
-    { &CONFIG_JOYSTICK,               "joystick",               1 },
-    { &CONFIG_JOYSTICK_DEVICE,        "joystick_device",        0 },
-    { &CONFIG_JOYSTICK_AXIS_X,        "joystick_axis_x",        0 },
-    { &CONFIG_JOYSTICK_AXIS_Y,        "joystick_axis_y",        1 },
-    { &CONFIG_JOYSTICK_AXIS_U,        "joystick_axis_u",        2 },
-    { &CONFIG_JOYSTICK_AXIS_X_INVERT, "joystick_axis_x_invert", 0 },
-    { &CONFIG_JOYSTICK_AXIS_Y_INVERT, "joystick_axis_y_invert", 0 },
-    { &CONFIG_JOYSTICK_AXIS_U_INVERT, "joystick_axis_u_invert", 0 },
+    { &CONFIG_JOYSTICK,                "joystick",                1 },
+    { &CONFIG_JOYSTICK_DEVICE,         "joystick_device",         0 },
+    { &CONFIG_JOYSTICK_RESPONSE,       "joystick_response",       250 },
+    { &CONFIG_JOYSTICK_AXIS_X0,        "joystick_axis_x0",        0 },
+    { &CONFIG_JOYSTICK_AXIS_Y0,        "joystick_axis_y0",        1 },
+    { &CONFIG_JOYSTICK_AXIS_X1,        "joystick_axis_x1",        2 },
+    { &CONFIG_JOYSTICK_AXIS_Y1,        "joystick_axis_y1",        3 },
+    { &CONFIG_JOYSTICK_AXIS_X0_INVERT, "joystick_axis_x0_invert", 0 },
+    { &CONFIG_JOYSTICK_AXIS_Y0_INVERT, "joystick_axis_y0_invert", 0 },
+    { &CONFIG_JOYSTICK_AXIS_X1_INVERT, "joystick_axis_x1_invert", 0 },
+    { &CONFIG_JOYSTICK_AXIS_Y1_INVERT, "joystick_axis_y1_invert", 0 },
+
     { &CONFIG_JOYSTICK_BUTTON_A,      "joystick_button_a",      0 },
     { &CONFIG_JOYSTICK_BUTTON_B,      "joystick_button_b",      1 },
-    { &CONFIG_JOYSTICK_BUTTON_R,      "joystick_button_r",      2 },
-    { &CONFIG_JOYSTICK_BUTTON_L,      "joystick_button_l",      3 },
-    { &CONFIG_JOYSTICK_BUTTON_EXIT,   "joystick_button_exit",   4 },
-    { &CONFIG_JOYSTICK_CAMERA_1,      "joystick_camera_1",      5 },
-    { &CONFIG_JOYSTICK_CAMERA_2,      "joystick_camera_2",      6 },
-    { &CONFIG_JOYSTICK_CAMERA_3,      "joystick_camera_3",      7 },
-    { &CONFIG_JOYSTICK_DPAD_L,        "joystick_dpad_l",        8 },
-    { &CONFIG_JOYSTICK_DPAD_R,        "joystick_dpad_r",        9 },
-    { &CONFIG_JOYSTICK_DPAD_U,        "joystick_dpad_u",        10 },
-    { &CONFIG_JOYSTICK_DPAD_D,        "joystick_dpad_d",        11 },
-    { &CONFIG_JOYSTICK_CAMERA_TOGGLE, "joystick_camera_toggle", 12 },
-    { &CONFIG_JOYSTICK_ROTATE_FAST,   "joystick_rotate_fast",   13 },
+    { &CONFIG_JOYSTICK_BUTTON_X,      "joystick_button_x",      2 },
+    { &CONFIG_JOYSTICK_BUTTON_Y,      "joystick_button_y",      3 },
+    { &CONFIG_JOYSTICK_BUTTON_L1,     "joystick_button_l1",     4 },
+    { &CONFIG_JOYSTICK_BUTTON_R1,     "joystick_button_r1",     5 },
+    { &CONFIG_JOYSTICK_BUTTON_L2,     "joystick_button_l2",     6 },
+    { &CONFIG_JOYSTICK_BUTTON_R2,     "joystick_button_r2",     7 },
+    { &CONFIG_JOYSTICK_BUTTON_START,  "joystick_button_start",  8 },
+    { &CONFIG_JOYSTICK_DPAD_L,        "joystick_dpad_l",       -1 },
+    { &CONFIG_JOYSTICK_DPAD_R,        "joystick_dpad_r",       -1 },
+    { &CONFIG_JOYSTICK_DPAD_U,        "joystick_dpad_u",       -1 },
+    { &CONFIG_JOYSTICK_DPAD_D,        "joystick_dpad_d",       -1 },
 
-    { &CONFIG_KEY_CAMERA_1,      "key_camera_1",      SDLK_F1 },
-    { &CONFIG_KEY_CAMERA_2,      "key_camera_2",      SDLK_F2 },
-    { &CONFIG_KEY_CAMERA_3,      "key_camera_3",      SDLK_F3 },
+    { &CONFIG_KEY_CAMERA_1,      "key_camera_1",      SDLK_1 },
+    { &CONFIG_KEY_CAMERA_2,      "key_camera_2",      SDLK_2 },
+    { &CONFIG_KEY_CAMERA_3,      "key_camera_3",      SDLK_3 },
     { &CONFIG_KEY_CAMERA_TOGGLE, "key_camera_toggle", SDLK_e },
     { &CONFIG_KEY_CAMERA_R,      "key_camera_r",      SDLK_d },
     { &CONFIG_KEY_CAMERA_L,      "key_camera_l",      SDLK_s },
@@ -177,7 +198,6 @@ static struct
     { &CONFIG_KEY_BACKWARD,      "key_backward",      SDLK_DOWN },
     { &CONFIG_KEY_LEFT,          "key_left",          SDLK_LEFT },
     { &CONFIG_KEY_RIGHT,         "key_right",         SDLK_RIGHT },
-    { &CONFIG_KEY_PAUSE,         "key_pause",         SDLK_ESCAPE },
     { &CONFIG_KEY_RESTART,       "key_restart",       SDLK_r },
     { &CONFIG_KEY_SCORE_NEXT,    "key_score_next",    SDLK_TAB },
     { &CONFIG_KEY_ROTATE_FAST,   "key_rotate_fast",   SDLK_LSHIFT },
@@ -191,7 +211,11 @@ static struct
     { &CONFIG_CHEAT,       "cheat",       0 },
     { &CONFIG_STATS,       "stats",       0 },
     { &CONFIG_SCREENSHOT,  "screenshot",  0 },
-    { &CONFIG_LOCK_GOALS,  "lock_goals",  0 }
+    { &CONFIG_LOCK_GOALS,  "lock_goals",  0 },
+
+    { &CONFIG_CAMERA_1_SPEED, "camera_1_speed", 250 },
+    { &CONFIG_CAMERA_2_SPEED, "camera_2_speed", 0 },
+    { &CONFIG_CAMERA_3_SPEED, "camera_3_speed", -1 },
 };
 
 static struct
@@ -204,7 +228,9 @@ static struct
     { &CONFIG_PLAYER,       "player",       "" },
     { &CONFIG_BALL_FILE,    "ball_file",    "ball/basic-ball/basic-ball" },
     { &CONFIG_WIIMOTE_ADDR, "wiimote_addr", "" },
-    { &CONFIG_REPLAY_NAME,  "replay_name",  "%s-%l" }
+    { &CONFIG_REPLAY_NAME,  "replay_name",  "%s-%l" },
+    { &CONFIG_LANGUAGE,     "language",     "" },
+    { &CONFIG_THEME,        "theme",        "classic" }
 };
 
 static int dirty = 0;
@@ -213,16 +239,12 @@ static int dirty = 0;
 
 static void config_key(const char *s, int i)
 {
-    int c;
+    SDL_Keycode c = SDL_GetKeyFromName(s);
 
-    config_set_d(i, option_d[i].def);
-
-    for (c = 0; c < SDLK_LAST; c++)
-        if (strcmp(s, SDL_GetKeyName((SDLKey) c)) == 0)
-        {
-            config_set_d(i, c);
-            break;
-        }
+    if (c == SDLK_UNKNOWN)
+        config_set_d(i, option_d[i].def);
+    else
+        config_set_d(i, c);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -235,12 +257,8 @@ static void config_mouse(const char *s, int i)
         config_set_d(i, SDL_BUTTON_LEFT);
     else if (strcmp(s, "right") == 0)
         config_set_d(i, SDL_BUTTON_RIGHT);
-    else if (strcmp(s, "wheelup") == 0)
-        config_set_d(i, SDL_BUTTON_WHEELUP);
     else if (strcmp(s, "middle") == 0)
         config_set_d(i, SDL_BUTTON_MIDDLE);
-    else if (strcmp(s, "wheeldown") == 0)
-        config_set_d(i, SDL_BUTTON_WHEELDOWN);
     else
         config_set_d(i, atoi(s));
 }
@@ -256,9 +274,7 @@ static const char *config_mouse_name(int b)
     case 0:                    return "none";      break;
     case SDL_BUTTON_LEFT:      return "left";      break;
     case SDL_BUTTON_RIGHT:     return "right";     break;
-    case SDL_BUTTON_WHEELUP:   return "wheelup";   break;
     case SDL_BUTTON_MIDDLE:    return "middle";    break;
-    case SDL_BUTTON_WHEELDOWN: return "wheeldown"; break;
     default:                   return buff;        break;
     }
 }
@@ -326,6 +342,8 @@ void config_load(void)
 {
     fs_file fh;
 
+    SDL_assert(SDL_WasInit(SDL_INIT_VIDEO));
+
     if ((fh = fs_open(USER_CONFIG_FILE, "r")))
     {
         char *line, *key, *val;
@@ -363,7 +381,6 @@ void config_load(void)
                                  i == CONFIG_KEY_CAMERA_TOGGLE ||
                                  i == CONFIG_KEY_CAMERA_R      ||
                                  i == CONFIG_KEY_CAMERA_L      ||
-                                 i == CONFIG_KEY_PAUSE         ||
                                  i == CONFIG_KEY_RESTART       ||
                                  i == CONFIG_KEY_SCORE_NEXT    ||
                                  i == CONFIG_KEY_ROTATE_FAST)
@@ -402,6 +419,8 @@ void config_save(void)
 {
     fs_file fh;
 
+    SDL_assert(SDL_WasInit(SDL_INIT_VIDEO));
+
     if (dirty && (fh = fs_open(USER_CONFIG_FILE, "w")))
     {
         int i;
@@ -433,12 +452,11 @@ void config_save(void)
                      i == CONFIG_KEY_CAMERA_TOGGLE ||
                      i == CONFIG_KEY_CAMERA_R      ||
                      i == CONFIG_KEY_CAMERA_L      ||
-                     i == CONFIG_KEY_PAUSE         ||
                      i == CONFIG_KEY_RESTART       ||
                      i == CONFIG_KEY_SCORE_NEXT    ||
                      i == CONFIG_KEY_ROTATE_FAST)
             {
-                s = SDL_GetKeyName((SDLKey) option_d[i].cur);
+                s = SDL_GetKeyName((SDL_Keycode) option_d[i].cur);
             }
             else if (i == CONFIG_CHEAT)
             {
@@ -455,10 +473,7 @@ void config_save(void)
         /* Write out string options. */
 
         for (i = 0; i < ARRAYSIZE(option_s); i++)
-        {
-            if (option_s[i].cur && *option_s[i].cur)
-                fs_printf(fh, "%-25s %s\n", option_s[i].name, option_s[i].cur);
-        }
+            fs_printf(fh, "%-25s %s\n", option_s[i].name, option_s[i].cur);
 
         fs_close(fh);
     }
