@@ -23,6 +23,7 @@
 #include "image.h"
 #include "base_image.h"
 #include "config.h"
+#include "video.h"
 
 #include "fs.h"
 #include "fs_png.h"
@@ -36,8 +37,8 @@ void image_snap(const char *filename)
     png_infop   infop  = NULL;
     png_bytep  *bytep  = NULL;
 
-    int w = config_get_d(CONFIG_WIDTH);
-    int h = config_get_d(CONFIG_HEIGHT);
+    int w = video.device_w;
+    int h = video.device_h;
     int i;
 
     unsigned char *p = NULL;
@@ -59,7 +60,7 @@ void image_snap(const char *filename)
 
         png_set_write_fn(writep, filep, fs_png_write, fs_png_flush);
         png_set_IHDR(writep, infop, w, h, 8,
-                     PNG_COLOR_TYPE_RGB_ALPHA,
+                     PNG_COLOR_TYPE_RGB,
                      PNG_INTERLACE_NONE,
                      PNG_COMPRESSION_TYPE_DEFAULT,
                      PNG_FILTER_TYPE_DEFAULT);
@@ -77,12 +78,12 @@ void image_snap(const char *filename)
                 for (i = 0; i < h; ++i)
                     bytep[h - i - 1] = (png_bytep) (p + i * w * 4);
 
-                png_set_rows (writep, infop, bytep);
-
                 /* Write the PNG image file. */
 
                 png_write_info(writep, infop);
-                png_write_png (writep, infop, 0, NULL);
+                png_set_filler(writep, 0, PNG_FILLER_AFTER);
+                png_write_image(writep, bytep);
+                png_write_end(writep, infop);
 
                 free(bytep);
             }
@@ -101,13 +102,10 @@ void image_snap(const char *filename)
 /*
  * Create an OpenGL texture object using the given image buffer.
  */
-static GLuint make_texture(const void *p, int w, int h, int b)
+GLuint make_texture(const void *p, int w, int h, int b, int fl)
 {
     static const GLenum format[] =
         { 0, GL_LUMINANCE, GL_LUMINANCE_ALPHA, GL_RGB, GL_RGBA };
-    static const GLenum type[] =
-        { 0, GL_UNSIGNED_BYTE, GL_UNSIGNED_BYTE,
-            GL_UNSIGNED_SHORT_5_6_5, GL_UNSIGNED_SHORT_4_4_4_4 };
 
     GLuint o = 0;
 
@@ -117,7 +115,7 @@ static GLuint make_texture(const void *p, int w, int h, int b)
     int a = config_get_d(CONFIG_ANISO);
 #endif
 #ifdef GL_GENERATE_MIPMAP_SGIS
-    int m = config_get_d(CONFIG_MIPMAP);
+    int m = (fl & IF_MIPMAP) ? config_get_d(CONFIG_MIPMAP) : 0;
 #endif
     int k = config_get_d(CONFIG_TEXTURES);
     int W = w;
@@ -160,7 +158,7 @@ static GLuint make_texture(const void *p, int w, int h, int b)
 
     glTexImage2D(GL_TEXTURE_2D, 0,
                  format[b], W, H, 0,
-                 format[b], type[b], q ? q : p);
+                 format[b], GL_UNSIGNED_BYTE, q ? q : p);
 
     if (q) free(q);
 
@@ -168,45 +166,10 @@ static GLuint make_texture(const void *p, int w, int h, int b)
     return o;
 }
 
-static void *image_to_16bpp(unsigned char *data, int w, int h, int b)
-{
-    unsigned char *p, *row;
-    int i, j;
-
-    if (b <= 2) return data;
-
-    if ((p = malloc(w * h * 2)))
-    {
-        for (i = 0; i < h; i++)
-        {
-            row = data + i * w * b;
-            for (j = 0; j < w; j++)
-            {
-                unsigned short packed, *pp;
-                if (b == 4)
-                    packed =
-                        (row[j * b] >> 4) << 12 |
-                        (row[j * b + 1] >> 4) << 8 |
-                        (row[j * b + 2] >> 4) << 4 |
-                        row[j * b + 3] >> 4;
-                else
-                    packed =
-                        (row[j * b] >> 3) << 11 |
-                        (row[j * b + 1] >> 2) << 5 |
-                        (row[j * b + 2] >> 3);
-                pp = (unsigned short *)&p[i * w * 2 + j * 2];
-                *pp = packed;
-            }
-        }
-    }
-    free(data);
-    return p;
-}
-
 /*
  * Load an image from the named file.  Return an OpenGL texture object.
  */
-GLuint make_image_from_file(const char *filename)
+GLuint make_image_from_file(const char *filename, int fl)
 {
     void  *p;
     int    w;
@@ -218,8 +181,7 @@ GLuint make_image_from_file(const char *filename)
 
     if ((p = image_load(filename, &w, &h, &b)))
     {
-        p = image_to_16bpp(p, w, h, b);
-        o = make_texture(p, w, h, b);
+        o = make_texture(p, w, h, b, fl);
         free(p);
     }
 
@@ -235,7 +197,7 @@ GLuint make_image_from_file(const char *filename)
  */
 GLuint make_image_from_font(int *W, int *H,
                             int *w, int *h,
-                            const char *text, TTF_Font *font)
+                            const char *text, TTF_Font *font, int fl)
 {
     GLuint o = 0;
 
@@ -265,9 +227,6 @@ GLuint make_image_from_font(int *W, int *H,
 
             if ((src = SDL_ConvertSurface(orig, &fmt, orig->flags)) == NULL)
             {
-                fprintf(stderr, _("Failed to convert SDL_ttf surface: %s\n"),
-                        SDL_GetError());
-
                 /* Pretend everything's just fine. */
 
                 src = orig;
@@ -290,8 +249,7 @@ GLuint make_image_from_font(int *W, int *H,
 
             /* Create the OpenGL texture object. */
 
-            p = image_to_16bpp(p, w2, h2, b);
-            o = make_texture(p, w2, h2, b);
+            o = make_texture(p, w2, h2, b, fl);
 
             free(p);
             SDL_FreeSurface(src);
@@ -329,8 +287,10 @@ SDL_Surface *load_surface(const char *filename)
         void *q;
 
         if ((q = image_flip(p, w, h, b, 0, 1)))
+        {
             srf = SDL_CreateRGBSurfaceFrom(q, w, h, b * 8, w * b,
                                            RMASK, GMASK, BMASK, AMASK);
+        }
         free(p);
     }
     return srf;
