@@ -25,11 +25,13 @@
 
 enum
 {
-    SOL_VER_MINIMUM = 6,
-    SOL_VER_GLES,
-    SOL_VER_CURRENT = SOL_VER_GLES
+    SOL_VERSION_1_5 = 6,
+    SOL_VERSION_DEV
 };
 
+#define SOL_VERSION_MIN  SOL_VERSION_1_5
+#define SOL_VERSION_CURR SOL_VERSION_DEV
+
 #define SOL_MAGIC (0xAF | 'S' << 8 | 'O' << 16 | 'L' << 24)
 
 /*---------------------------------------------------------------------------*/
@@ -41,11 +43,11 @@ static int sol_file(fs_file fin)
     int magic;
     int version;
 
-    get_index(fin, &magic);
-    get_index(fin, &version);
+    magic   = get_index(fin);
+    version = get_index(fin);
 
-    if (magic != SOL_MAGIC || (version < SOL_VER_MINIMUM ||
-                               version > SOL_VER_CURRENT))
+    if (magic != SOL_MAGIC || (version < SOL_VERSION_MIN ||
+                               version > SOL_VERSION_CURR))
         return 0;
 
     sol_version = version;
@@ -55,16 +57,28 @@ static int sol_file(fs_file fin)
 
 static void sol_load_mtrl(fs_file fin, struct b_mtrl *mp)
 {
-    get_array(fin,  mp->d, 4);
-    get_array(fin,  mp->a, 4);
-    get_array(fin,  mp->s, 4);
-    get_array(fin,  mp->e, 4);
-    get_array(fin,  mp->h, 1);
-    get_index(fin, &mp->fl);
+    get_array(fin, mp->d, 4);
+    get_array(fin, mp->a, 4);
+    get_array(fin, mp->s, 4);
+    get_array(fin, mp->e, 4);
+    get_array(fin, mp->h, 1);
+
+    mp->fl = get_index(fin);
 
     fs_read(mp->f, 1, PATHMAX, fin);
 
-    if (sol_version < SOL_VER_GLES)
+    if (sol_version >= SOL_VERSION_DEV)
+    {
+        if (mp->fl & M_ALPHA_TEST)
+        {
+            mp->alpha_func = get_index(fin);
+            mp->alpha_ref  = get_float(fin);
+        }
+    }
+
+    /* Convert 1.5.4 material flags. */
+
+    if (sol_version == SOL_VERSION_1_5)
     {
         static const int flags[][2] = {
             { 1, M_SHADOWED },
@@ -73,12 +87,10 @@ static void sol_load_mtrl(fs_file fin, struct b_mtrl *mp)
             { 8, M_ENVIRONMENT },
             { 16, M_ADDITIVE },
             { 32, M_CLAMP_S | M_CLAMP_T },
-            { 64, M_DECAL },
+            { 64, M_DECAL | M_SHADOWED },
             { 128, M_TWO_SIDED }
         };
 
-        /* Convert 1.5.4 material flags. */
-
         if (mp->fl)
         {
             int i, f;
@@ -101,42 +113,43 @@ static void sol_load_mtrl(fs_file fin, struct b_mtrl *mp)
 
 static void sol_load_vert(fs_file fin, struct b_vert *vp)
 {
-    get_array(fin,  vp->p, 3);
+    get_array(fin, vp->p, 3);
 }
 
 static void sol_load_edge(fs_file fin, struct b_edge *ep)
 {
-    get_index(fin, &ep->vi);
-    get_index(fin, &ep->vj);
+    ep->vi = get_index(fin);
+    ep->vj = get_index(fin);
 }
 
 static void sol_load_side(fs_file fin, struct b_side *sp)
 {
-    get_array(fin,  sp->n, 3);
-    get_float(fin, &sp->d);
+    get_array(fin, sp->n, 3);
+
+    sp->d = get_float(fin);
 }
 
 static void sol_load_texc(fs_file fin, struct b_texc *tp)
 {
-    get_array(fin,  tp->u, 2);
+    get_array(fin, tp->u, 2);
 }
 
 static void sol_load_offs(fs_file fin, struct b_offs *op)
 {
-    get_index(fin, &op->ti);
-    get_index(fin, &op->si);
-    get_index(fin, &op->vi);
+    op->ti = get_index(fin);
+    op->si = get_index(fin);
+    op->vi = get_index(fin);
 }
 
 static void sol_load_geom(fs_file fin, struct b_geom *gp, struct s_base *fp)
 {
-    get_index(fin, &gp->mi);
+    gp->mi = get_index(fin);
 
-    if (sol_version >= SOL_VER_GLES)
+    if (sol_version >= SOL_VERSION_DEV)
     {
-        get_index(fin, &gp->oi);
-        get_index(fin, &gp->oj);
-        get_index(fin, &gp->ok);
+        gp->oi = get_index(fin);
+        gp->oj = get_index(fin);
+        gp->ok = get_index(fin);
     }
     else
     {
@@ -148,9 +161,9 @@ static void sol_load_geom(fs_file fin, struct b_geom *gp, struct s_base *fp)
 
         for (i = 0; i < 3; i++)
         {
-            get_index(fin, &ov[i].ti);
-            get_index(fin, &ov[i].si);
-            get_index(fin, &ov[i].vi);
+            ov[i].ti = get_index(fin);
+            ov[i].si = get_index(fin);
+            ov[i].vi = get_index(fin);
 
             iv[i] = -1;
 
@@ -187,39 +200,40 @@ static void sol_load_geom(fs_file fin, struct b_geom *gp, struct s_base *fp)
 
 static void sol_load_lump(fs_file fin, struct b_lump *lp)
 {
-    get_index(fin, &lp->fl);
-    get_index(fin, &lp->v0);
-    get_index(fin, &lp->vc);
-    get_index(fin, &lp->e0);
-    get_index(fin, &lp->ec);
-    get_index(fin, &lp->g0);
-    get_index(fin, &lp->gc);
-    get_index(fin, &lp->s0);
-    get_index(fin, &lp->sc);
+    lp->fl = get_index(fin);
+    lp->v0 = get_index(fin);
+    lp->vc = get_index(fin);
+    lp->e0 = get_index(fin);
+    lp->ec = get_index(fin);
+    lp->g0 = get_index(fin);
+    lp->gc = get_index(fin);
+    lp->s0 = get_index(fin);
+    lp->sc = get_index(fin);
 }
 
 static void sol_load_node(fs_file fin, struct b_node *np)
 {
-    get_index(fin, &np->si);
-    get_index(fin, &np->ni);
-    get_index(fin, &np->nj);
-    get_index(fin, &np->l0);
-    get_index(fin, &np->lc);
+    np->si = get_index(fin);
+    np->ni = get_index(fin);
+    np->nj = get_index(fin);
+    np->l0 = get_index(fin);
+    np->lc = get_index(fin);
 }
 
 static void sol_load_path(fs_file fin, struct b_path *pp)
 {
-    get_array(fin,  pp->p, 3);
-    get_float(fin, &pp->t);
-    get_index(fin, &pp->pi);
-    get_index(fin, &pp->f);
-    get_index(fin, &pp->s);
+    get_array(fin, pp->p, 3);
+
+    pp->t  = get_float(fin);
+    pp->pi = get_index(fin);
+    pp->f  = get_index(fin);
+    pp->s  = get_index(fin);
 
     pp->tm = TIME_TO_MS(pp->t);
     pp->t  = MS_TO_TIME(pp->tm);
 
-    if (sol_version >= SOL_VER_GLES)
-        get_index(fin, &pp->fl);
+    if (sol_version >= SOL_VERSION_DEV)
+        pp->fl = get_index(fin);
 
     pp->e[0] = 1.0f;
     pp->e[1] = 0.0f;
@@ -232,40 +246,51 @@ static void sol_load_path(fs_file fin, struct b_path *pp)
 
 static void sol_load_body(fs_file fin, struct b_body *bp)
 {
-    get_index(fin, &bp->pi);
-    get_index(fin, &bp->ni);
-    get_index(fin, &bp->l0);
-    get_index(fin, &bp->lc);
-    get_index(fin, &bp->g0);
-    get_index(fin, &bp->gc);
+    bp->pi = get_index(fin);
+
+    if (sol_version >= SOL_VERSION_DEV)
+    {
+        bp->pj = get_index(fin);
+
+        if (bp->pj < 0)
+            bp->pj = bp->pi;
+    }
+    else
+        bp->pj = bp->pi;
+
+    bp->ni = get_index(fin);
+    bp->l0 = get_index(fin);
+    bp->lc = get_index(fin);
+    bp->g0 = get_index(fin);
+    bp->gc = get_index(fin);
 }
 
 static void sol_load_item(fs_file fin, struct b_item *hp)
 {
-    get_array(fin,  hp->p, 3);
-    get_index(fin, &hp->t);
-    get_index(fin, &hp->n);
+    get_array(fin, hp->p, 3);
+
+    hp->t = get_index(fin);
+    hp->n = get_index(fin);
 }
 
 static void sol_load_goal(fs_file fin, struct b_goal *zp)
 {
-    get_array(fin,  zp->p, 3);
-    get_float(fin, &zp->r);
+    get_array(fin, zp->p, 3);
+
+    zp->r = get_float(fin);
 }
 
 static void sol_load_swch(fs_file fin, struct b_swch *xp)
 {
-    float f;
-    int i;
+    get_array(fin, xp->p, 3);
 
-    get_array(fin,  xp->p, 3);
-    get_float(fin, &xp->r);
-    get_index(fin, &xp->pi);
-    get_float(fin, &xp->t);
-    get_float(fin, &f);
-    get_index(fin, &xp->f);
-    get_index(fin, &i);
-    get_index(fin, &xp->i);
+    xp->r  = get_float(fin);
+    xp->pi = get_index(fin);
+    xp->t  = get_float(fin);
+    (void)   get_float(fin);
+    xp->f  = get_index(fin);
+    (void)   get_index(fin);
+    xp->i  = get_index(fin);
 
     xp->tm = TIME_TO_MS(xp->t);
     xp->t = MS_TO_TIME(xp->tm);
@@ -273,69 +298,72 @@ static void sol_load_swch(fs_file fin, struct b_swch *xp)
 
 static void sol_load_bill(fs_file fin, struct b_bill *rp)
 {
-    get_index(fin, &rp->fl);
-    get_index(fin, &rp->mi);
-    get_float(fin, &rp->t);
-    get_float(fin, &rp->d);
-    get_array(fin,  rp->w,  3);
-    get_array(fin,  rp->h,  3);
-    get_array(fin,  rp->rx, 3);
-    get_array(fin,  rp->ry, 3);
-    get_array(fin,  rp->rz, 3);
-    get_array(fin,  rp->p,  3);
+    rp->fl = get_index(fin);
+    rp->mi = get_index(fin);
+    rp->t  = get_float(fin);
+    rp->d  = get_float(fin);
+
+    get_array(fin, rp->w,  3);
+    get_array(fin, rp->h,  3);
+    get_array(fin, rp->rx, 3);
+    get_array(fin, rp->ry, 3);
+    get_array(fin, rp->rz, 3);
+    get_array(fin, rp->p,  3);
 }
 
 static void sol_load_jump(fs_file fin, struct b_jump *jp)
 {
-    get_array(fin,  jp->p, 3);
-    get_array(fin,  jp->q, 3);
-    get_float(fin, &jp->r);
+    get_array(fin, jp->p, 3);
+    get_array(fin, jp->q, 3);
+
+    jp->r = get_float(fin);
 }
 
 static void sol_load_ball(fs_file fin, struct b_ball *up)
 {
-    get_array(fin,  up->p, 3);
-    get_float(fin, &up->r);
+    get_array(fin, up->p, 3);
+
+    up->r = get_float(fin);
 }
 
 static void sol_load_view(fs_file fin, struct b_view *wp)
 {
-    get_array(fin,  wp->p, 3);
-    get_array(fin,  wp->q, 3);
+    get_array(fin, wp->p, 3);
+    get_array(fin, wp->q, 3);
 }
 
 static void sol_load_dict(fs_file fin, struct b_dict *dp)
 {
-    get_index(fin, &dp->ai);
-    get_index(fin, &dp->aj);
+    dp->ai = get_index(fin);
+    dp->aj = get_index(fin);
 }
 
 static void sol_load_indx(fs_file fin, struct s_base *fp)
 {
-    get_index(fin, &fp->ac);
-    get_index(fin, &fp->dc);
-    get_index(fin, &fp->mc);
-    get_index(fin, &fp->vc);
-    get_index(fin, &fp->ec);
-    get_index(fin, &fp->sc);
-    get_index(fin, &fp->tc);
+    fp->ac = get_index(fin);
+    fp->dc = get_index(fin);
+    fp->mc = get_index(fin);
+    fp->vc = get_index(fin);
+    fp->ec = get_index(fin);
+    fp->sc = get_index(fin);
+    fp->tc = get_index(fin);
 
-    if (sol_version >= SOL_VER_GLES)
-        get_index(fin, &fp->oc);
+    if (sol_version >= SOL_VERSION_DEV)
+        fp->oc = get_index(fin);
 
-    get_index(fin, &fp->gc);
-    get_index(fin, &fp->lc);
-    get_index(fin, &fp->nc);
-    get_index(fin, &fp->pc);
-    get_index(fin, &fp->bc);
-    get_index(fin, &fp->hc);
-    get_index(fin, &fp->zc);
-    get_index(fin, &fp->jc);
-    get_index(fin, &fp->xc);
-    get_index(fin, &fp->rc);
-    get_index(fin, &fp->uc);
-    get_index(fin, &fp->wc);
-    get_index(fin, &fp->ic);
+    fp->gc = get_index(fin);
+    fp->lc = get_index(fin);
+    fp->nc = get_index(fin);
+    fp->pc = get_index(fin);
+    fp->bc = get_index(fin);
+    fp->hc = get_index(fin);
+    fp->zc = get_index(fin);
+    fp->jc = get_index(fin);
+    fp->xc = get_index(fin);
+    fp->rc = get_index(fin);
+    fp->uc = get_index(fin);
+    fp->wc = get_index(fin);
+    fp->ic = get_index(fin);
 }
 
 static int sol_load_file(fs_file fin, struct s_base *fp)
@@ -412,7 +440,7 @@ static int sol_load_file(fs_file fin, struct s_base *fp)
     for (i = 0; i < fp->rc; i++) sol_load_bill(fin, fp->rv + i);
     for (i = 0; i < fp->uc; i++) sol_load_ball(fin, fp->uv + i);
     for (i = 0; i < fp->wc; i++) sol_load_view(fin, fp->wv + i);
-    for (i = 0; i < fp->ic; i++) get_index(fin, fp->iv + i);
+    for (i = 0; i < fp->ic; i++) fp->iv[i] = get_index(fin);
 
     /* Magically "fix" all of our code. */
 
@@ -471,6 +499,8 @@ int sol_load_meta(struct s_base *fp, const char *filename)
     fs_file fin;
     int res = 0;
 
+    memset(fp, 0, sizeof (*fp));
+
     if ((fin = fs_open(filename, "r")))
     {
         res = sol_load_head(fin, fp);
@@ -518,6 +548,12 @@ static void sol_stor_mtrl(fs_file fout, struct b_mtrl *mp)
     put_index(fout, mp->fl);
 
     fs_write(mp->f, 1, PATHMAX, fout);
+
+    if (mp->fl & M_ALPHA_TEST)
+    {
+        put_index(fout, mp->alpha_func);
+        put_float(fout, mp->alpha_ref);
+    }
 }
 
 static void sol_stor_vert(fs_file fout, struct b_vert *vp)
@@ -595,6 +631,7 @@ static void sol_stor_path(fs_file fout, struct b_path *pp)
 static void sol_stor_body(fs_file fout, struct b_body *bp)
 {
     put_index(fout, bp->pi);
+    put_index(fout, bp->pj);
     put_index(fout, bp->ni);
     put_index(fout, bp->l0);
     put_index(fout, bp->lc);
@@ -670,7 +707,7 @@ static void sol_stor_file(fs_file fout, struct s_base *fp)
 {
     int i;
     int magic   = SOL_MAGIC;
-    int version = SOL_VER_CURRENT;
+    int version = SOL_VERSION_CURR;
 
     put_index(fout, magic);
     put_index(fout, version);
@@ -736,3 +773,162 @@ int sol_stor_base(struct s_base *fp, const char *filename)
 }
 
 /*---------------------------------------------------------------------------*/
+
+const struct path tex_paths[4] = {
+    { "textures/", ".png" },
+    { "textures/", ".jpg" },
+    { "",          ".png" },
+    { "",          ".jpg" }
+};
+
+const struct path mtrl_paths[2] = {
+    { "textures/", "" },
+    { "",          "" }
+};
+
+/*---------------------------------------------------------------------------*/
+
+/*
+ * This has to match up with mtrl_func_syms in mtrl.c.
+ */
+static const char mtrl_func_names[8][16] = {
+    "always",
+    "equal",
+    "gequal",
+    "greater",
+    "lequal",
+    "less",
+    "never",
+    "notequal"
+};
+
+static const struct
+{
+    char name[16];
+    int flag;
+} mtrl_flags[] = {
+    { "additive",    M_ADDITIVE },
+    { "clamp-s",     M_CLAMP_S },
+    { "clamp-t",     M_CLAMP_T },
+    { "decal",       M_DECAL },
+    { "environment", M_ENVIRONMENT },
+    { "reflective",  M_REFLECTIVE },
+    { "shadowed",    M_SHADOWED },
+    { "transparent", M_TRANSPARENT },
+    { "two-sided",   M_TWO_SIDED },
+    { "particle",    M_PARTICLE },
+};
+
+int mtrl_read(struct b_mtrl *mp, const char *name)
+{
+    static char line[MAXSTR];
+    static char word[MAXSTR];
+
+    fs_file fp;
+    int i;
+
+    if (mp && name && *name)
+    {
+        SAFECPY(mp->f, name);
+
+        mp->a[0] = mp->a[1] = mp->a[2] = 0.2f;
+        mp->d[0] = mp->d[1] = mp->d[2] = 0.8f;
+        mp->s[0] = mp->s[1] = mp->s[2] = 0.0f;
+        mp->e[0] = mp->e[1] = mp->e[2] = 0.0f;
+        mp->a[3] = mp->d[3] = mp->s[3] = mp->e[3] = 1.0f;
+        mp->h[0] = 0.0f;
+        mp->fl   = 0;
+        mp->angle = 45.0f;
+
+        mp->alpha_func = 0;
+        mp->alpha_ref  = 0.0f;
+
+        fp = NULL;
+
+        for (i = 0; i < ARRAYSIZE(mtrl_paths); i++)
+        {
+            CONCAT_PATH(line, &mtrl_paths[i], name);
+
+            if ((fp = fs_open(line, "r")))
+                break;
+        }
+
+        if (fp)
+        {
+            char str[16] = "";
+
+            while (fs_gets(line, sizeof (line), fp))
+            {
+                char *p = strip_newline(line);
+
+                if (sscanf(p, "diffuse %f %f %f %f",
+                           &mp->d[0], &mp->d[1],
+                           &mp->d[2], &mp->d[3]) == 4)
+                {
+                }
+                else if (sscanf(p, "ambient %f %f %f %f",
+                                &mp->a[0], &mp->a[1],
+                                &mp->a[2], &mp->a[3]) == 4)
+                {
+                }
+                else if (sscanf(p, "specular %f %f %f %f",
+                                &mp->s[0], &mp->s[1],
+                                &mp->s[2], &mp->s[3]) == 4)
+                {
+                }
+                else if (sscanf(p, "emissive %f %f %f %f",
+                                &mp->e[0], &mp->e[1],
+                                &mp->e[2], &mp->e[3]) == 4)
+                {
+                }
+                else if (sscanf(p, "shininess %f", &mp->h[0]) == 1)
+                {
+                }
+                else if (strncmp(p, "flags ", 6) == 0)
+                {
+                    int f = 0;
+                    int n;
+
+                    p += 6;
+
+                    while (sscanf(p, "%s%n", word, &n) > 0)
+                    {
+                        for (i = 0; i < ARRAYSIZE(mtrl_flags); i++)
+                            if (strcmp(word, mtrl_flags[i].name) == 0)
+                            {
+                                f |= mtrl_flags[i].flag;
+                                break;
+                            }
+
+                        p += n;
+                    }
+
+                    mp->fl = f;
+                }
+                else if (sscanf(p, "angle %f", &mp->angle) == 1)
+                {
+                }
+                else if (sscanf(p, "alpha-test %15s %f",
+                                str, &mp->alpha_ref) == 2)
+                {
+                    mp->fl |= M_ALPHA_TEST;
+
+                    for (i = 0; i < ARRAYSIZE(mtrl_func_names); i++)
+                        if (strcmp(str, mtrl_func_names[i]) == 0)
+                        {
+                            mp->alpha_func = i;
+                            break;
+                        }
+                }
+                else /* Unknown directive */;
+            }
+
+            fs_close(fp);
+            return 1;
+        }
+        else /* Unknown material */;
+    }
+    return 0;
+}
+
+/*---------------------------------------------------------------------------*/
