@@ -21,10 +21,12 @@
 #include <math.h>
 
 #include "glext.h"
+#include "video.h"
 #include "vec3.h"
 #include "geom.h"
 #include "image.h"
 #include "base_image.h"
+#include "config.h"
 #include "base_config.h"
 #include "lang.h"
 
@@ -33,24 +35,20 @@
 
 /*---------------------------------------------------------------------------*/
 
-/* EXCLUDED material flags for each rendering pass. */
+/*
+ * Included and excluded material flags for each rendering pass.
+ */
 
-static const int pass_ex[] = {
-    M_REFLECTIVE | M_TRANSPARENT | M_DECAL,
-    M_REFLECTIVE | M_TRANSPARENT,
-    M_REFLECTIVE,
-    M_REFLECTIVE | M_DECAL,
-    0,
-};
-
-/* INCLUDED material flags for each rendering pass. */
-
-static const int pass_in[] = {
-    0,
-    M_DECAL,
-    M_DECAL | M_TRANSPARENT,
-    M_TRANSPARENT,
-    M_REFLECTIVE,
+static const struct
+{
+    int in;
+    int ex;
+} passes[PASS_MAX] = {
+    { 0,                       M_REFLECTIVE | M_TRANSPARENT | M_DECAL },
+    { M_DECAL,                 M_REFLECTIVE | M_TRANSPARENT },
+    { M_DECAL | M_TRANSPARENT, M_REFLECTIVE },
+    { M_TRANSPARENT,           M_REFLECTIVE | M_DECAL },
+    { M_REFLECTIVE,            0 }
 };
 
 /*---------------------------------------------------------------------------*/
@@ -65,13 +63,16 @@ static void sol_transform(const struct s_vary *vary,
 
     /* Apply the body position and rotation to the model-view matrix. */
 
-    sol_body_p(p, vary, bp->pi, bp->t);
-    sol_body_e(e, vary, bp, 0);
+    sol_body_p(p, vary, bp, 0.0f);
+    sol_body_e(e, vary, bp, 0.0f);
 
     q_as_axisangle(e, v, &a);
 
-    glTranslatef(p[0], p[1], p[2]);
-    glRotatef(V_DEG(a), v[0], v[1], v[2]);
+    if (!(p[0] == 0 && p[1] == 0 && p[2] == 0))
+        glTranslatef(p[0], p[1], p[2]);
+
+    if (!((v[0] == 0 && v[1] == 0 && v[2] == 0) || a == 0))
+        glRotatef(V_DEG(a), v[0], v[1], v[2]);
 
     /* Apply the shadow transform to the texture matrix. */
 
@@ -132,11 +133,6 @@ static void sol_transform(const struct s_vary *vary,
 static void sol_load_bill(struct s_draw *draw)
 {
     static const GLfloat data[] = {
-        0.0f,  0.0f, -1.0f, -1.0f,
-        1.0f,  0.0f,  1.0f, -1.0f,
-        0.0f,  1.0f, -1.0f,  1.0f,
-        1.0f,  1.0f,  1.0f,  1.0f,
-
         0.0f,  0.0f, -0.5f,  0.0f,
         1.0f,  0.0f,  0.5f,  0.0f,
         0.0f,  1.0f, -0.5f,  1.0f,
@@ -158,29 +154,22 @@ static void sol_load_bill(struct s_draw *draw)
 
 static void sol_free_bill(struct s_draw *draw)
 {
-    if (glIsBuffer_(draw->bill))
-        glDeleteBuffers_(1, &draw->bill);
+    glDeleteBuffers_(1, &draw->bill);
 }
 
-static void sol_draw_bill(GLfloat w, GLfloat h, GLboolean edge)
+static void sol_draw_bill(GLboolean edge)
 {
-    glPushMatrix();
-    {
-        glScalef(0.5f * w, 0.5f * h, 1.0f);
-
-        if (edge)
-            glTranslatef(0.0f, 0.5f, 0.0f);
-
+    if (edge)
         glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-    }
-    glPopMatrix();
+    else
+        glDrawArrays(GL_TRIANGLE_STRIP, 4, 4);
 }
 
 /*---------------------------------------------------------------------------*/
 
 /* NOTE: The state management here presumes that billboard rendering is      */
-/* NESTED within a wider SOL rendering process. That is: sol_draw_enable     */
-/* has been called and sol_draw_disable will be called in the future.        */
+/* NESTED within a wider SOL rendering process. That is: r_draw_enable       */
+/* has been called and r_draw_disable will be called in the future.          */
 /* Thus the "default" VBO state retained by billboard rendering is the       */
 /* state appropriate for normal SOL rendering.                               */
 
@@ -190,214 +179,29 @@ static void sol_bill_enable(const struct s_draw *draw)
 
     glBindBuffer_(GL_ARRAY_BUFFER, draw->bill);
 
+    glDisableClientState(GL_NORMAL_ARRAY);
+
     glTexCoordPointer(2, GL_FLOAT, s * 4, (GLvoid *) (    0));
     glVertexPointer  (2, GL_FLOAT, s * 4, (GLvoid *) (s * 2));
 }
 
 static void sol_bill_disable(void)
 {
+    glEnableClientState(GL_NORMAL_ARRAY);
+
+    glBindBuffer_(GL_ARRAY_BUFFER, 0);
 }
 
 /*---------------------------------------------------------------------------*/
 
-#define tobyte(f) ((GLubyte) (f * 255.0f))
-
-static struct b_mtrl default_base_mtrl =
+static int sol_test_mtrl(int mi, int p)
 {
-    { 0.8f, 0.8f, 0.8f, 1.0f },
-    { 0.2f, 0.2f, 0.2f, 1.0f },
-    { 0.0f, 0.0f, 0.0f, 1.0f },
-    { 0.0f, 0.0f, 0.0f, 1.0f },
-    { 0.0f }, 0.0f, 0, ""
-};
+    const struct mtrl *mp = mtrl_get(mi);
 
-/* Nasty. */
+    /* Test whether the material flags match inclusion rules. */
 
-static struct d_mtrl default_draw_mtrl =
-{
-    &default_base_mtrl,
-    0xffcccccc,
-    0xff333333,
-    0xff000000,
-    0xff000000,
-    0x00000000,
-    0
-};
-
-void sol_apply_mtrl(const struct d_mtrl *mp_draw, struct s_rend *rend)
-{
-    const struct b_mtrl *mp_base = mp_draw->base;
-    const struct d_mtrl *mq_draw = rend->mp;
-    const struct b_mtrl *mq_base = mq_draw->base;
-
-    /* Bind the texture. */
-
-    if (mp_draw->o != mq_draw->o)
-        glBindTexture(GL_TEXTURE_2D, mp_draw->o);
-
-    /* Set material properties. */
-
-    if (mp_draw->d != mq_draw->d)
-        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   mp_base->d);
-    if (mp_draw->a != mq_draw->a)
-        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   mp_base->a);
-    if (mp_draw->s != mq_draw->s)
-        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  mp_base->s);
-    if (mp_draw->e != mq_draw->e)
-        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  mp_base->e);
-    if (mp_draw->h != mq_draw->h)
-        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mp_base->h);
-
-    /* Ball shadow. */
-
-    if ((mp_base->fl & M_SHADOWED) ^ (mq_base->fl & M_SHADOWED))
-    {
-        if (mp_base->fl & M_SHADOWED)
-            shad_draw_set();
-        else
-            shad_draw_clr();
-    }
-
-    /* Environment mapping. */
-
-#if !ENABLE_OPENGLES
-    if ((mp_base->fl & M_ENVIRONMENT) ^ (mq_base->fl & M_ENVIRONMENT))
-    {
-        if (mp_base->fl & M_ENVIRONMENT)
-        {
-            glEnable(GL_TEXTURE_GEN_S);
-            glEnable(GL_TEXTURE_GEN_T);
-
-            glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
-            glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
-        }
-        else
-        {
-            glDisable(GL_TEXTURE_GEN_S);
-            glDisable(GL_TEXTURE_GEN_T);
-        }
-    }
-#endif
-
-    /* Additive blending. */
-
-    if ((mp_base->fl & M_ADDITIVE) ^ (mq_base->fl & M_ADDITIVE))
-    {
-        if (mp_base->fl & M_ADDITIVE)
-            glBlendFunc(GL_ONE, GL_ONE);
-        else
-            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-    }
-
-    /* Visibility-from-behind. */
-
-    if ((mp_base->fl & M_TWO_SIDED) ^ (mq_base->fl & M_TWO_SIDED))
-    {
-        if (mp_base->fl & M_TWO_SIDED)
-        {
-            glDisable(GL_CULL_FACE);
-            glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1);
-        }
-        else
-        {
-            glEnable(GL_CULL_FACE);
-            glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 0);
-        }
-    }
-
-    /* Decal offset. */
-
-    if ((mp_base->fl & M_DECAL) ^ (mq_base->fl & M_DECAL))
-    {
-        if (mp_base->fl & M_DECAL)
-        {
-            glEnable(GL_POLYGON_OFFSET_FILL);
-            glPolygonOffset(-1.0f, -2.0f);
-        }
-        else
-            glDisable(GL_POLYGON_OFFSET_FILL);
-    }
-
-    rend->mp = mp_draw;
-}
-
-static GLuint sol_find_texture(const char *name)
-{
-    char png[MAXSTR];
-    char jpg[MAXSTR];
-
-    GLuint o;
-
-    /* Prefer a lossless copy of the texture over a lossy compression. */
-
-    strncpy(png, name, PATHMAX); strcat(png, ".png");
-    strncpy(jpg, name, PATHMAX); strcat(jpg, ".jpg");
-
-    /* Check for a PNG. */
-
-    if ((o = make_image_from_file(png)))
-        return o;
-
-    /* Check for a JPG. */
-
-    if ((o = make_image_from_file(jpg)))
-        return o;
-
-    return 0;
-}
-
-void sol_load_mtrl(struct d_mtrl *mp, const struct b_mtrl *mq)
-{
-    mp->base = mq;
-
-    if ((mp->o = sol_find_texture(_(mq->f))))
-    {
-        /* Set the texture to clamp or repeat based on material type. */
-
-        if (mq->fl & M_CLAMP_S)
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-        else
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-
-        if (mq->fl & M_CLAMP_T)
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-        else
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-    }
-
-    /* Cache the 32-bit material values for quick comparison. */
-
-    mp->d = (tobyte(mq->d[0]))
-        |   (tobyte(mq->d[1]) <<  8)
-        |   (tobyte(mq->d[2]) << 16)
-        |   (tobyte(mq->d[3]) << 24);
-    mp->a = (tobyte(mq->a[0]))
-        |   (tobyte(mq->a[1]) <<  8)
-        |   (tobyte(mq->a[2]) << 16)
-        |   (tobyte(mq->a[3]) << 24);
-    mp->s = (tobyte(mq->s[0]))
-        |   (tobyte(mq->s[1]) <<  8)
-        |   (tobyte(mq->s[2]) << 16)
-        |   (tobyte(mq->s[3]) << 24);
-    mp->e = (tobyte(mq->e[0]))
-        |   (tobyte(mq->e[1]) <<  8)
-        |   (tobyte(mq->e[2]) << 16)
-        |   (tobyte(mq->e[3]) << 24);
-    mp->h = (tobyte(mq->h[0]));
-}
-
-void sol_free_mtrl(struct d_mtrl *mp)
-{
-    if (glIsTexture(mp->o))
-        glDeleteTextures(1, &mp->o);
-}
-
-static int sol_test_mtrl(const struct d_mtrl *mp, int p)
-{
-    /* Test whether the material flags exclude f0 and include f1. */
-
-    return ((mp->base->fl & pass_in[p]) == pass_in[p] &&
-            (mp->base->fl & pass_ex[p]) == 0);
+    return ((mp->base.fl & passes[p].in) == passes[p].in &&
+            (mp->base.fl & passes[p].ex) == 0);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -442,7 +246,7 @@ static int sol_count_mesh(const struct d_body *bp, int p)
     /* Count the body meshes matching the given material flags. */
 
     for (mi = 0; mi < bp->mc; ++mi)
-        if (sol_test_mtrl(bp->mv[mi].mp, p))
+        if (sol_test_mtrl(bp->mv[mi].mtrl, p))
             c++;
 
     return c;
@@ -566,8 +370,12 @@ static void sol_load_mesh(struct d_mesh *mp,
         glBufferData_(GL_ELEMENT_ARRAY_BUFFER, gn * gs, gv, GL_STATIC_DRAW);
         glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER, 0);
 
-        mp->mp  = draw->mv + mi;
+        /* Note cached material index. */
+
+        mp->mtrl = draw->base->mtrls[mi];
+
         mp->ebc = gn * 3;
+        mp->vbc = vn;
     }
 
     free(iv);
@@ -577,24 +385,22 @@ static void sol_load_mesh(struct d_mesh *mp,
 
 static void sol_free_mesh(struct d_mesh *mp)
 {
-    if (glIsBuffer_(mp->ebo))
-        glDeleteBuffers_(1, &mp->ebo);
-    if (glIsBuffer_(mp->vbo))
-        glDeleteBuffers_(1, &mp->vbo);
+    glDeleteBuffers_(1, &mp->ebo);
+    glDeleteBuffers_(1, &mp->vbo);
 }
 
 void sol_draw_mesh(const struct d_mesh *mp, struct s_rend *rend, int p)
 {
     /* If this mesh has material matching the given flags... */
 
-    if (sol_test_mtrl(mp->mp, p))
+    if (sol_test_mtrl(mp->mtrl, p))
     {
         const size_t s = sizeof (struct d_vert);
         const GLenum T = GL_FLOAT;
 
         /* Apply the material state. */
 
-        sol_apply_mtrl(mp->mp, rend);
+        r_apply_mtrl(rend, mp->mtrl);
 
         /* Bind the mesh data. */
 
@@ -617,7 +423,10 @@ void sol_draw_mesh(const struct d_mesh *mp, struct s_rend *rend, int p)
 
         /* Draw the mesh. */
 
-        glDrawElements(GL_TRIANGLES, mp->ebc, GL_UNSIGNED_SHORT, 0);
+        if (rend->curr_mtrl.base.fl & M_PARTICLE)
+            glDrawArrays(GL_POINTS, 0, mp->vbc);
+        else
+            glDrawElements(GL_TRIANGLES, mp->ebc, GL_UNSIGNED_SHORT, 0);
     }
 }
 
@@ -634,7 +443,7 @@ static void sol_load_body(struct d_body *bp,
 
     /* Determine how many materials this body uses. */
 
-    for (mi = 0; mi < draw->mc; ++mi)
+    for (mi = 0; mi < draw->base->mc; ++mi)
         if (sol_count_body(bq, draw->base, mi))
             bp->mc++;
 
@@ -644,7 +453,7 @@ static void sol_load_body(struct d_body *bp,
     {
         int mj = 0;
 
-        for (mi = 0; mi < draw->mc; ++mi)
+        for (mi = 0; mi < draw->base->mc; ++mi)
             if (sol_count_body(bq, draw->base, mi))
                 sol_load_mesh(bp->mv + mj++, bq, draw, mi);
     }
@@ -653,8 +462,8 @@ static void sol_load_body(struct d_body *bp,
 
     bp->pass[0] = sol_count_mesh(bp, 0);
     bp->pass[1] = sol_count_mesh(bp, 1);
-    bp->pass[2] = sol_count_mesh(bp, 2); 
-    bp->pass[3] = sol_count_mesh(bp, 3); 
+    bp->pass[2] = sol_count_mesh(bp, 2);
+    bp->pass[3] = sol_count_mesh(bp, 3);
     bp->pass[4] = sol_count_mesh(bp, 4);
 }
 
@@ -678,7 +487,7 @@ static void sol_draw_body(const struct d_body *bp, struct s_rend *rend, int p)
 
 /*---------------------------------------------------------------------------*/
 
-int sol_load_draw(struct s_draw *draw, const struct s_vary *vary, int s)
+int sol_load_draw(struct s_draw *draw, struct s_vary *vary, int s)
 {
     int i;
 
@@ -687,29 +496,23 @@ int sol_load_draw(struct s_draw *draw, const struct s_vary *vary, int s)
     draw->vary = vary;
     draw->base = vary->base;
 
-    /* Initialize all materials for this file. */
+    /* Determine whether this file has reflective materials. */
 
-    if (draw->base->mc)
-    {
-        if ((draw->mv = calloc(draw->base->mc, sizeof (*draw->mv))))
+    for (i = 0; i < draw->base->mc; i++)
+        if (draw->base->mv[i].fl & M_REFLECTIVE)
         {
-            draw->mc = draw->base->mc;
-
-            for (i = 0; i < draw->mc; i++)
-            {
-                sol_load_mtrl(draw->mv + i, draw->base->mv + i);
-
-                /* If at least one material is reflective, mark it. */
-
-                if (draw->base->mv[i].fl & M_REFLECTIVE)
-                    draw->reflective = 1;
-            }
+            draw->reflective = 1;
+            break;
         }
-    }
+
+    /* Cache all materials for this file. */
+
+    mtrl_cache_sol(draw->base);
 
     /* Initialize shadow state. */
 
     draw->shadow_ui = -1;
+    draw->shadowed = s;
 
     /* Initialize all bodies for this file. */
 
@@ -733,12 +536,14 @@ void sol_free_draw(struct s_draw *draw)
 {
     int i;
 
+    mtrl_free_sol(draw->base);
+
     sol_free_bill(draw);
 
     for (i = 0; i < draw->bc; i++)
         sol_free_body(draw->bv + i);
-    for (i = 0; i < draw->mc; i++)
-        sol_free_mtrl(draw->mv + i);
+
+    free(draw->bv);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -750,7 +555,6 @@ static void sol_draw_all(const struct s_draw *draw, struct s_rend *rend, int p)
     /* Draw all meshes of all bodies matching the given material flags. */
 
     for (bi = 0; bi < draw->bc; ++bi)
-
         if (draw->bv[bi].pass[p])
         {
             glPushMatrix();
@@ -762,71 +566,26 @@ static void sol_draw_all(const struct s_draw *draw, struct s_rend *rend, int p)
         }
 }
 
-void sol_draw_enable(struct s_rend *rend)
-{
-    glEnableClientState(GL_VERTEX_ARRAY);
-    glEnableClientState(GL_NORMAL_ARRAY);
-
-#ifndef __MAEMO__
-    if (gli.max_texture_units > 2)
-        tex_env_active(&tex_env_shadow_clip);
-    else if (gli.max_texture_units > 1)
-        tex_env_active(&tex_env_shadow);
-    else
-#endif
-        tex_env_active(&tex_env_default);
-
-    if (tex_env_stage(TEX_STAGE_SHADOW))
-    {
-        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-        if (tex_env_stage(TEX_STAGE_CLIP))
-            glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-        tex_env_stage(TEX_STAGE_TEXTURE);
-    }
-    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-
-    rend->mp = &default_draw_mtrl;
-}
-
-void sol_draw_disable(struct s_rend *rend)
-{
-    sol_apply_mtrl(&default_draw_mtrl, rend);
-
-    if (tex_env_stage(TEX_STAGE_SHADOW))
-    {
-        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-
-        if (tex_env_stage(TEX_STAGE_CLIP))
-            glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-
-        tex_env_stage(TEX_STAGE_TEXTURE);
-    }
-    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-
-    tex_env_active(&tex_env_default);
-
-    glDisableClientState(GL_NORMAL_ARRAY);
-    glDisableClientState(GL_VERTEX_ARRAY);
-}
-
 /*---------------------------------------------------------------------------*/
 
 void sol_draw(const struct s_draw *draw, struct s_rend *rend, int mask, int test)
 {
+    /* Disable shadowed material setup if not requested. */
+
+    rend->skip_flags |= (draw->shadowed ? 0 : M_SHADOWED);
+
     /* Render all opaque geometry, decals last. */
 
-    sol_draw_all(draw, rend, 0);
-    sol_draw_all(draw, rend, 1);
+    sol_draw_all(draw, rend, PASS_OPAQUE);
+    sol_draw_all(draw, rend, PASS_OPAQUE_DECAL);
 
     /* Render all transparent geometry, decals first. */
 
     if (!test) glDisable(GL_DEPTH_TEST);
     if (!mask) glDepthMask(GL_FALSE);
     {
-        sol_draw_all(draw, rend, 2);
-        sol_draw_all(draw, rend, 3);
+        sol_draw_all(draw, rend, PASS_TRANSPARENT_DECAL);
+        sol_draw_all(draw, rend, PASS_TRANSPARENT);
     }
     if (!mask) glDepthMask(GL_TRUE);
     if (!test) glEnable(GL_DEPTH_TEST);
@@ -835,25 +594,33 @@ void sol_draw(const struct s_draw *draw, struct s_rend *rend, int mask, int test
 
     glBindBuffer_(GL_ARRAY_BUFFER,         0);
     glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER, 0);
+
+    rend->skip_flags = 0;
 }
 
 void sol_refl(const struct s_draw *draw, struct s_rend *rend)
 {
+    /* Disable shadowed material setup if not requested. */
+
+    rend->skip_flags |= (draw->shadowed ? 0 : M_SHADOWED);
+
     /* Render all reflective geometry. */
 
-    sol_draw_all(draw, rend, 4);
+    sol_draw_all(draw, rend, PASS_REFLECTIVE);
 
     /* Revert the buffer object state. */
 
     glBindBuffer_(GL_ARRAY_BUFFER,         0);
     glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER, 0);
+
+    rend->skip_flags = 0;
 }
 
 void sol_back(const struct s_draw *draw,
               struct s_rend *rend,
               float n, float f, float t)
 {
-    if (!draw || !draw->base)
+    if (!(draw && draw->base && draw->base->rc))
         return;
 
     glDisable(GL_LIGHTING);
@@ -886,6 +653,8 @@ void sol_back(const struct s_draw *draw,
                     float ry = rp->ry[0] + rp->ry[1] * T + rp->ry[2] * T * T;
                     float rz = rp->rz[0] + rp->rz[1] * T + rp->rz[2] * T * T;
 
+                    r_apply_mtrl(rend, draw->base->mtrls[rp->mi]);
+
                     glPushMatrix();
                     {
                         if (ry) glRotatef(ry, 0.0f, 1.0f, 0.0f);
@@ -905,12 +674,7 @@ void sol_back(const struct s_draw *draw,
 
                         glScalef(w, h, 1.0f);
 
-                        sol_apply_mtrl(draw->mv + rp->mi, rend);
-
-                        if (rp->fl & B_EDGE)
-                            glDrawArrays(GL_TRIANGLE_STRIP, 4, 4);
-                        else
-                            glDrawArrays(GL_TRIANGLE_STRIP, 8, 4);
+                        sol_draw_bill(rp->fl & B_EDGE);
                     }
                     glPopMatrix();
                 }
@@ -926,6 +690,9 @@ void sol_back(const struct s_draw *draw,
 void sol_bill(const struct s_draw *draw,
               struct s_rend *rend, const float *M, float t)
 {
+    if (!(draw && draw->base && draw->base->rc))
+        return;
+
     sol_bill_enable(draw);
     {
         int ri;
@@ -943,7 +710,7 @@ void sol_bill(const struct s_draw *draw,
             float ry = rp->ry[0] + rp->ry[1] * T + rp->ry[2] * S;
             float rz = rp->rz[0] + rp->rz[1] * T + rp->rz[2] * S;
 
-            sol_apply_mtrl(draw->mv + rp->mi, rend);
+            r_apply_mtrl(rend, draw->base->mtrls[rp->mi]);
 
             glPushMatrix();
             {
@@ -955,7 +722,9 @@ void sol_bill(const struct s_draw *draw,
                 if (fabsf(ry) > 0.0f) glRotatef(ry, 0.0f, 1.0f, 0.0f);
                 if (fabsf(rz) > 0.0f) glRotatef(rz, 0.0f, 0.0f, 1.0f);
 
-                sol_draw_bill(w, h, GL_FALSE);
+                glScalef(w, h, 1.0f);
+
+                sol_draw_bill(GL_FALSE);
             }
             glPopMatrix();
         }
@@ -963,7 +732,7 @@ void sol_bill(const struct s_draw *draw,
     sol_bill_disable();
 }
 
-void sol_fade(const struct s_draw *draw, float k)
+void sol_fade(const struct s_draw *draw, struct s_rend *rend, float k)
 {
     if (k > 0.0f)
     {
@@ -974,7 +743,6 @@ void sol_fade(const struct s_draw *draw, float k)
         glPushMatrix();
         glLoadIdentity();
         {
-            glEnable(GL_COLOR_MATERIAL);
             glDisable(GL_LIGHTING);
             glDisable(GL_DEPTH_TEST);
             glDisable(GL_TEXTURE_2D);
@@ -982,7 +750,9 @@ void sol_fade(const struct s_draw *draw, float k)
             glColor4f(0.0f, 0.0f, 0.0f, k);
 
             sol_bill_enable(draw);
-            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+            r_apply_mtrl(rend, default_mtrl);
+            glScalef(2.0f, 2.0f, 1.0f);
+            sol_draw_bill(GL_FALSE);
             sol_bill_disable();
 
             glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
@@ -990,7 +760,6 @@ void sol_fade(const struct s_draw *draw, float k)
             glEnable(GL_TEXTURE_2D);
             glEnable(GL_DEPTH_TEST);
             glEnable(GL_LIGHTING);
-            glDisable(GL_COLOR_MATERIAL);
         }
         glMatrixMode(GL_PROJECTION);
         glPopMatrix();
@@ -1003,12 +772,17 @@ void sol_fade(const struct s_draw *draw, float k)
 
 int sol_load_full(struct s_full *full, const char *filename, int s)
 {
-    if (sol_load_base(&full->base, filename))
+    if (full)
     {
-        sol_load_vary(&full->vary, &full->base);
-        sol_load_draw(&full->draw, &full->vary, s);
+        memset(full, 0, sizeof (*full));
 
-        return 1;
+        if (sol_load_base(&full->base, filename))
+        {
+            sol_load_vary(&full->vary, &full->base);
+            sol_load_draw(&full->draw, &full->vary, s);
+
+            return 1;
+        }
     }
 
     return 0;
@@ -1022,3 +796,242 @@ void sol_free_full(struct s_full *full)
 }
 
 /*---------------------------------------------------------------------------*/
+
+#if DEBUG_MTRL
+static void check_mtrl(const char *name, GLenum pname, GLuint curr)
+{
+    static char buff[64];
+
+    GLuint real;
+    GLfloat v[4];
+
+    glGetMaterialfv(GL_FRONT, pname, v);
+
+    if (pname != GL_SHININESS)
+        real = (tobyte(v[0])       |
+                tobyte(v[1]) << 8  |
+                tobyte(v[2]) << 16 |
+                tobyte(v[3]) << 24);
+    else
+        real = (tobyte(v[0]));
+
+    if (real != curr)
+    {
+        sprintf(buff, "%s mismatch (0x%08X -> 0x%08X)", name, real, curr);
+        glStringMarker_(buff);
+    }
+}
+
+static void assert_mtrl(const struct mtrl *mp)
+{
+    if (glIsEnabled(GL_COLOR_MATERIAL))
+        return;
+
+    check_mtrl("ambient",   GL_AMBIENT,   mp->a);
+    check_mtrl("diffuse",   GL_DIFFUSE,   mp->d);
+    check_mtrl("specular",  GL_SPECULAR,  mp->s);
+    check_mtrl("emission",  GL_EMISSION,  mp->e);
+    check_mtrl("shininess", GL_SHININESS, mp->h);
+}
+#endif
+
+void r_color_mtrl(struct s_rend *rend, int enable)
+{
+    if (enable)
+    {
+        glEnable(GL_COLOR_MATERIAL);
+
+        rend->color_mtrl = 1;
+    }
+    else
+    {
+        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
+
+        glDisable(GL_COLOR_MATERIAL);
+
+        /* Keep material tracking synchronized with GL state. */
+
+        rend->curr_mtrl.d = 0xffffffff;
+        rend->curr_mtrl.a = 0xffffffff;
+
+        rend->color_mtrl = 0;
+    }
+}
+
+void r_apply_mtrl(struct s_rend *rend, int mi)
+{
+    struct mtrl *mp = mtrl_get(mi);
+    struct mtrl *mq = &rend->curr_mtrl;
+
+    /* Mask ignored flags. */
+
+    int mp_flags = mp->base.fl & ~rend->skip_flags;
+    int mq_flags = mq->base.fl;
+
+#if DEBUG_MTRL
+    assert_mtrl(&rend->curr_mtrl);
+#endif
+
+    /* Bind the texture. */
+
+    if (mp->o != mq->o)
+        glBindTexture(GL_TEXTURE_2D, mp->o);
+
+    /* Set material properties. */
+
+    if (mp->d != mq->d && !rend->color_mtrl)
+        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,   mp->base.d);
+    if (mp->a != mq->a && !rend->color_mtrl)
+        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,   mp->base.a);
+    if (mp->s != mq->s)
+        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,  mp->base.s);
+    if (mp->e != mq->e)
+        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,  mp->base.e);
+    if (mp->h != mq->h)
+        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mp->base.h);
+
+    /* Ball shadow. */
+
+    if ((mp_flags & M_SHADOWED) ^ (mq_flags & M_SHADOWED))
+    {
+        if (mp_flags & M_SHADOWED)
+            shad_draw_set();
+        else
+            shad_draw_clr();
+    }
+
+    /* Environment mapping. */
+
+#if !ENABLE_OPENGLES
+    if ((mp_flags & M_ENVIRONMENT) ^ (mq_flags & M_ENVIRONMENT))
+    {
+        if (mp_flags & M_ENVIRONMENT)
+        {
+            glEnable(GL_TEXTURE_GEN_S);
+            glEnable(GL_TEXTURE_GEN_T);
+
+            glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
+            glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
+        }
+        else
+        {
+            glDisable(GL_TEXTURE_GEN_S);
+            glDisable(GL_TEXTURE_GEN_T);
+        }
+    }
+#endif
+
+    /* Additive blending. */
+
+    if ((mp_flags & M_ADDITIVE) ^ (mq_flags & M_ADDITIVE))
+    {
+        if (mp_flags & M_ADDITIVE)
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE);
+        else
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    }
+
+    /* Visibility-from-behind. */
+
+    if ((mp_flags & M_TWO_SIDED) ^ (mq_flags & M_TWO_SIDED))
+    {
+        if (mp_flags & M_TWO_SIDED)
+        {
+            glDisable(GL_CULL_FACE);
+            glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1);
+        }
+        else
+        {
+            glEnable(GL_CULL_FACE);
+            glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 0);
+        }
+    }
+
+    /* Decal offset. */
+
+    if ((mp_flags & M_DECAL) ^ (mq_flags & M_DECAL))
+    {
+        if (mp_flags & M_DECAL)
+        {
+            glEnable(GL_POLYGON_OFFSET_FILL);
+            glPolygonOffset(-1.0f, -2.0f);
+        }
+        else
+            glDisable(GL_POLYGON_OFFSET_FILL);
+    }
+
+    /* Alpha test. */
+
+    if ((mp_flags & M_ALPHA_TEST) ^ (mq_flags & M_ALPHA_TEST))
+    {
+        if (mp_flags & M_ALPHA_TEST)
+        {
+            glAlphaFunc(mtrl_func(mp->base.alpha_func), mp->base.alpha_ref);
+
+            glEnable(GL_ALPHA_TEST);
+        }
+        else
+            glDisable(GL_ALPHA_TEST);
+    }
+
+    if (((mp_flags & mq_flags) & M_ALPHA_TEST) && (mp->base.alpha_func !=
+                                                   mq->base.alpha_func ||
+                                                   mp->base.alpha_ref !=
+                                                   mq->base.alpha_ref))
+    {
+        /* Update alpha function. */
+
+        glAlphaFunc(mtrl_func(mp->base.alpha_func), mp->base.alpha_ref);
+    }
+
+    /* Point sprite. */
+
+    if ((mp_flags & M_PARTICLE) ^ (mq_flags & M_PARTICLE))
+    {
+        if (mp_flags & M_PARTICLE)
+        {
+            const int s = video.device_h / 4;
+            const GLfloat c[3] = { 0.0f, 0.0f, 1.0f };
+
+            glEnable (GL_POINT_SPRITE);
+            glTexEnvi(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE);
+            glPointParameterfv_(GL_POINT_DISTANCE_ATTENUATION, c);
+            glPointParameterf_ (GL_POINT_SIZE_MIN, 1);
+            glPointParameterf_ (GL_POINT_SIZE_MAX, s);
+        }
+        else
+        {
+            glDisable(GL_POINT_SPRITE);
+        }
+    }
+
+    /* Update current material state. */
+
+    memcpy(mq, mp, sizeof (struct mtrl));
+
+    mq->base.fl = mp_flags;
+}
+
+void r_draw_enable(struct s_rend *rend)
+{
+    memset(rend, 0, sizeof (*rend));
+
+    glEnableClientState(GL_VERTEX_ARRAY);
+    glEnableClientState(GL_NORMAL_ARRAY);
+    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+
+    glBindTexture(GL_TEXTURE_2D, 0);
+
+    rend->curr_mtrl = *mtrl_get(default_mtrl);
+}
+
+void r_draw_disable(struct s_rend *rend)
+{
+    r_apply_mtrl(rend, default_mtrl);
+
+    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    glDisableClientState(GL_NORMAL_ARRAY);
+    glDisableClientState(GL_VERTEX_ARRAY);
+}
+
+/*---------------------------------------------------------------------------*/
