@@ -20,7 +20,6 @@
 #include "solid_vary.h"
 #include "solid_sim.h"
 #include "solid_all.h"
-#include "solid_cmd.h"
 
 #define LARGE 1.0e+5f
 #define SMALL 1.0e-3f
@@ -63,6 +62,9 @@ static float v_sol(const float p[3], const float v[3], float r)
          Yet, the potential division by zero below seems fine.
     if (fabsf(a) < SMALL) return LARGE;
 */
+    /* Testing for equality against zero is acceptable. */
+
+    if (a == 0.0f) return LARGE;
 
     if      (d < 0.0f) return LARGE;
     else if (d > 0.0f)
@@ -213,7 +215,7 @@ static float v_side(float Q[3],
     float wn = v_dot(w, n);
     float t  = LARGE;
 
-    if (vn - wn <= 0.0f)
+    if (vn - wn < 0.0f)
     {
         float on = v_dot(o, n);
         float pn = v_dot(p, n);
@@ -270,7 +272,7 @@ static float sol_bounce(struct v_ball *up,
     vn = v_dot(v, n);
     wn = v_dot(w, n);
 
-    v_mad(v, v, n, 1.7 * (wn - vn));
+    v_mad(v, v, n, 1.7f * (wn - vn));
 
     v_mad(p, q, n, up->r);
 
@@ -514,14 +516,13 @@ static float sol_test_body(float dt,
                            const struct s_vary *vary,
                            const struct v_body *bp)
 {
-    float U[3], O[3], E[4], W[3], A[3], u;
+    float U[3], O[3], E[4], W[3], u;
 
     const struct b_node *np = vary->base->nv + bp->base->ni;
 
-    sol_body_p(O, vary, bp->pi, bp->t);
-    sol_body_v(W, vary, bp->pi, bp->t, dt);
-    sol_body_e(E, vary, bp, 0);
-    sol_body_w(A, vary, bp);
+    sol_body_p(O, vary, bp, 0.0f);
+    sol_body_v(W, vary, bp, dt);
+    sol_body_e(E, vary, bp, 0.0f);
 
     /*
      * For rotating bodies, rather than rotate every normal and vertex
@@ -534,7 +535,7 @@ static float sol_test_body(float dt,
      * v = w x p
      */
 
-    if (E[0] != 1.0f || v_dot(A, A) != 0.0f)
+    if (E[0] != 1.0f || sol_body_w(vary, bp))
     {
         /* The body has a non-identity orientation or it is rotating. */
 
@@ -568,13 +569,9 @@ static float sol_test_body(float dt,
 
         if ((u = sol_test_node(dt, U, &ball, vary->base, np, z, z)) < dt)
         {
-            float d[4];
-
             /* Compute the final orientation. */
 
-            q_by_axisangle(d, A, v_len(A) * u);
-            q_mul(e, E, d);
-            q_nrm(e, e);
+            sol_body_e(e, vary, bp, u);
 
             /* Return world space coordinates. */
 
@@ -630,49 +627,97 @@ static float sol_test_file(float dt,
 /*---------------------------------------------------------------------------*/
 
 /*
- * Track simulation steps in integer milliseconds.
+ * Accumulate and convert simulation time to integer milliseconds.
  */
 
-static float ms_accum;
-
-static void ms_init(void)
+static void ms_init(float *accum)
 {
-    ms_accum = 0.0f;
+    *accum = 0.0f;
 }
 
-static int ms_step(float dt)
+static int ms_step(float *accum, float dt)
 {
     int ms = 0;
 
-    ms_accum += dt;
+    *accum += dt;
 
-    while (ms_accum >= 0.001f)
+    while (*accum >= 0.001f)
     {
-        ms_accum -= 0.001f;
+        *accum -= 0.001f;
         ms += 1;
     }
 
     return ms;
 }
 
-static int ms_peek(float dt)
+static int ms_peek(float *accum, float dt)
 {
-    int ms = 0;
-    float at;
+    float at = *accum;
 
-    at = ms_accum + dt;
-
-    while (at >= 0.001f)
-    {
-        at -= 0.001f;
-        ms += 1;
-    }
-
-    return ms;
+    return ms_step(&at, dt);
 }
 
 /*---------------------------------------------------------------------------*/
 
+/*
+ * Find time till the next path change.
+ */
+static float sol_path_time(struct s_vary *vary, float dt)
+{
+    int mi;
+
+    for (mi = 0; mi < vary->mc; mi++)
+    {
+        struct v_move *mp = vary->mv + mi;
+        struct v_path *pp = vary->pv + mp->pi;
+
+        if (!pp->f)
+            continue;
+
+        if (mp->tm + ms_peek(&vary->ms_accum, dt) > pp->base->tm)
+            dt = MS_TO_TIME(pp->base->tm - mp->tm);
+    }
+
+    return dt;
+}
+
+/*
+ * Move SOL state forward DT seconds.
+ */
+static void sol_move_once(struct s_vary *vary, cmd_fn cmd_func, float dt)
+{
+    int ms;
+
+    if (cmd_func)
+    {
+        union cmd cmd = { CMD_STEP_SIMULATION };
+        cmd.stepsim.dt = dt;
+        cmd_func(&cmd);
+    }
+
+    ms = ms_step(&vary->ms_accum, dt);
+
+    sol_move_step(vary, cmd_func, dt, ms);
+    sol_swch_step(vary, cmd_func, dt, ms);
+    sol_ball_step(vary, cmd_func, dt);
+}
+
+/*
+ * Move SOL state forward DT seconds across multiple path changes.
+ */
+void sol_move(struct s_vary *vary, cmd_fn cmd_func, float dt)
+{
+    if (vary && vary->base)
+    {
+        while (dt > 0.0f)
+        {
+            float pt = sol_path_time(vary, dt);
+            sol_move_once(vary, cmd_func, pt);
+            dt -= pt;
+        }
+    }
+}
+
 /*
  * Step the physics forward DT  seconds under the influence of gravity
  * vector G.  If the ball gets pinched between two moving solids, this
@@ -681,13 +726,12 @@ static int ms_peek(float dt)
  * iterations, punt it.
  */
 
-float sol_step(struct s_vary *vary, const float *g, float dt, int ui, int *m)
+float sol_step(struct s_vary *vary, cmd_fn cmd_func,
+               const float *g, float dt, int ui, int *m)
 {
-    float P[3], V[3], v[3], r[3], a[3], d, e, nt, b = 0.0f, tt = dt;
+    float P[3], V[3], v[3], r[3], a[3], d, nt, b = 0.0f, tt = dt;
     int c;
 
-    union cmd cmd;
-
     if (ui < vary->uc)
     {
         struct v_ball *up = vary->uv + ui;
@@ -705,12 +749,13 @@ float sol_step(struct s_vary *vary, const float *g, float dt, int ui, int *m)
 
             if ((d = v_dot(r, g) / (v_len(r) * v_len(g))) > 0.999f)
             {
-                if ((e = (v_len(up->v) - dt)) > 0.0f)
+                if (v_len(up->v) > dt)
                 {
                     /* Scale the linear velocity. */
 
-                    v_nrm(up->v, up->v);
-                    v_scl(up->v, up->v, e);
+                    v_sub(v, V, up->v);
+                    v_nrm(v, v);
+                    v_mad(up->v, up->v, v, dt);
 
                     /* Scale the angular velocity. */
 
@@ -737,47 +782,22 @@ float sol_step(struct s_vary *vary, const float *g, float dt, int ui, int *m)
 
         for (c = 16; c > 0 && tt > 0; c--)
         {
-            float st;
-            int bi, ms;
+            float pt;
 
-            /* HACK: avoid stepping across path changes. */
+            /* Avoid stepping across path changes. */
 
-            st = tt;
-
-            for (bi = 0; bi < vary->bc; bi++)
-            {
-                struct v_body *bp = vary->bv + bi;
-
-                if (bp->pi >= 0)
-                {
-                    struct v_path *pp = vary->pv + bp->pi;
-
-                    if (!pp->f)
-                        continue;
-
-                    if (bp->tm + ms_peek(st) > pp->base->tm)
-                        st = MS_TO_TIME(pp->base->tm - bp->tm);
-                }
-            }
+            pt = sol_path_time(vary, tt);
 
             /* Miss collisions if we reach the iteration limit. */
 
             if (c > 1)
-                nt = sol_test_file(st, P, V, up, vary);
+                nt = sol_test_file(pt, P, V, up, vary);
             else
                 nt = tt;
 
-            cmd.type       = CMD_STEP_SIMULATION;
-            cmd.stepsim.dt = nt;
-            sol_cmd_enq(&cmd);
+            sol_move_once(vary, cmd_func, nt);
 
-            ms = ms_step(nt);
-
-            sol_body_step(vary, nt, ms);
-            sol_swch_step(vary, nt, ms);
-            sol_ball_step(vary, nt);
-
-            if (nt < st)
+            if (nt < pt)
                 if (b < (d = sol_bounce(up, P, V, nt)))
                     b = d;
 
@@ -796,7 +816,7 @@ float sol_step(struct s_vary *vary, const float *g, float dt, int ui, int *m)
 
 void sol_init_sim(struct s_vary *vary)
 {
-    ms_init();
+    ms_init(&vary->ms_accum);
 }
 
 void sol_quit_sim(void)
