@@ -277,26 +277,6 @@ void m_vxfm(float *v, const float *M, const float *w)
 
 /*---------------------------------------------------------------------------*/
 
-void m_view(float *M,
-            const float *c,
-            const float *p,
-            const float *u)
-{
-    float x[3];
-    float y[3];
-    float z[3];
-
-    v_sub(z, p, c);
-    v_nrm(z, z);
-    v_crs(x, u, z);
-    v_nrm(x, x);
-    v_crs(y, z, x);
-
-    m_basis(M, x, y, z);
-}
-
-/*---------------------------------------------------------------------------*/
-
 void q_as_axisangle(const float q[4], float u[3], float *a)
 {
     *a = 2.0f * facosf(q[0]);
@@ -418,12 +398,10 @@ void q_slerp(float q[4], const float a[4], const float b[4], float t)
 
     /* Fall back to normalized lerp for very similar orientations. */
 
-    if (1.0f - c < TINY)
+    if (1.0f - c < (float) TINY)
     {
         u = 1.0f - t;
         v = t;
-
-        i = 1;
     }
     else
     {
@@ -431,8 +409,6 @@ void q_slerp(float q[4], const float a[4], const float b[4], float t)
         s = fsinf(r);
         u = fsinf((1.0f - t) * r) / s;
         v = fsinf((t)        * r) / s * i;
-
-        i = 0;
     }
 
     q[0] = a[0] * u + b[0] * v;
@@ -440,7 +416,7 @@ void q_slerp(float q[4], const float a[4], const float b[4], float t)
     q[2] = a[2] * u + b[2] * v;
     q[3] = a[3] * u + b[3] * v;
 
-    if (i) q_nrm(q, q);
+    q_nrm(q, q);
 }
 
 /*---------------------------------------------------------------------------*/
